@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 # Copyright 2016 Yelp Inc.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -16,11 +15,13 @@
 Kafka checks module.
 Each check is separated subcommand for kafka-check.
 """
-from __future__ import absolute_import
-from __future__ import print_function
+from __future__ import annotations
 
 import argparse
 import logging
+from typing import Any
+from typing import cast
+from typing import Dict
 
 from kafka_utils.kafka_check import status_code
 from kafka_utils.kafka_check.commands.min_isr import MinIsrCmd
@@ -34,9 +35,9 @@ from kafka_utils.util import config
 from kafka_utils.util.error import ConfigurationError
 
 
-def convert_to_broker_id(string):
+def convert_to_broker_id(string: str) -> int:
     """Convert string to kafka broker_id."""
-    error_msg = 'Positive integer or -1 required, {string} given.'.format(string=string)
+    error_msg = f'Positive integer or -1 required, {string} given.'
     try:
         value = int(string)
     except ValueError:
@@ -46,7 +47,7 @@ def convert_to_broker_id(string):
     return value
 
 
-def parse_args():
+def parse_args() -> argparse.Namespace:
     """Parse the command line arguments."""
     parser = argparse.ArgumentParser(
         description='Check kafka current status',
@@ -110,6 +111,12 @@ def parse_args():
         action="store_true",
         default=False,
     )
+    parser.add_argument(
+        '--head',
+        help='Show N head lines of the output',
+        type=int,
+        default=-1,
+    )
 
     subparsers = parser.add_subparsers()
     MinIsrCmd().add_subparser(subparsers)
@@ -120,20 +127,13 @@ def parse_args():
     return parser.parse_args()
 
 
-def run():
-    """Verify command-line arguments and run commands"""
-    args = parse_args()
-    logging.basicConfig(level=logging.WARN)
-
-    # to prevent flooding for sensu-check.
-    logging.getLogger('kafka').setLevel(logging.CRITICAL)
-
+def validate_args(args: argparse.Namespace) -> None:
     if args.controller_only and args.first_broker_only:
         terminate(
             status_code.WARNING,
-            prepare_terminate_message(
+            cast(Dict[str, Any], prepare_terminate_message(
                 "Only one of controller_only and first_broker_only should be used",
-            ),
+            )),
             args.json,
         )
 
@@ -141,7 +141,7 @@ def run():
         if args.broker_id is None:
             terminate(
                 status_code.WARNING,
-                prepare_terminate_message("broker_id is not specified"),
+                cast(Dict[str, Any], prepare_terminate_message("broker_id is not specified")),
                 args.json,
             )
         elif args.broker_id == -1:
@@ -150,22 +150,39 @@ def run():
             except Exception as e:
                 terminate(
                     status_code.WARNING,
-                    prepare_terminate_message("{}".format(e)),
+                    cast(Dict[str, Any], prepare_terminate_message(f"{e}")),
                     args.json,
                 )
 
+    if args.head != -1 and not args.verbose:
+        terminate(
+            status_code.WARNING,
+            cast(Dict[str, Any], prepare_terminate_message("--head option works only as addition to --verbose option")),
+            args.json,
+        )
+
+
+def run() -> None:
+    """Verify command-line arguments and run commands"""
+    args = parse_args()
+    validate_args(args)
+
+    logging.basicConfig(level=logging.WARN)
+    # to prevent flooding for sensu-check.
+    logging.getLogger('kafka').setLevel(logging.CRITICAL)
+
     try:
         cluster_config = config.get_cluster_config(
             args.cluster_type,
             args.cluster_name,
             args.discovery_base_path,
         )
-        code, msg = args.command(cluster_config, args)
+        err_code, msg = args.command(cluster_config, args)
     except ConfigurationError as e:
         terminate(
             status_code.CRITICAL,
-            prepare_terminate_message("ConfigurationError {0}".format(e)),
+            cast(Dict[str, Any], prepare_terminate_message(f"ConfigurationError {e}")),
             args.json,
         )
 
-    terminate(code, msg, args.json)
+    terminate(err_code, msg, args.json)
