@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 # Copyright 2016 Yelp Inc.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -12,10 +11,12 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
-from __future__ import absolute_import
-from __future__ import print_function
+from __future__ import annotations
 
 import sys
+from typing import Any
+
+from typing_extensions import TypedDict
 
 from kafka_utils.util import print_json
 
@@ -30,25 +31,30 @@ STATUS_STRING = {
 }
 
 
-def prepare_terminate_message(string):
+class TerminateMessageDict(TypedDict):
+    message: str
+    raw: str
+
+
+def prepare_terminate_message(string: str) -> TerminateMessageDict:
     return {
         'message': string,
         'raw': string,
     }
 
 
-def terminate(signal, msg, json):
+def terminate(err_code: int, msg: dict[str, Any], json: bool) -> None:
     if json:
         output = {
-            'status': STATUS_STRING[signal],
+            'status': STATUS_STRING[err_code],
             'data': msg['raw'],
         }
         print_json(output)
     else:
         print('{status}: {msg}'.format(
-            status=STATUS_STRING[signal],
+            status=STATUS_STRING[err_code],
             msg=msg['message'],
         ))
         if 'verbose' in msg:
             print(msg['verbose'])
-    sys.exit(signal)
+    sys.exit(err_code)
