@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 # Copyright 2016 Yelp Inc.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -12,24 +11,23 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
-from __future__ import absolute_import
-from __future__ import division
-from __future__ import unicode_literals
+from __future__ import annotations
 
 import logging
 from collections import OrderedDict
-
-import six
+from typing import Any
+from typing import Callable
 
 from .broker import Broker
 from .error import InvalidBrokerIdError
 from .error import InvalidPartitionError
 from .partition import Partition
+from .partition_measurer import PartitionMeasurer
 from .rg import ReplicationGroup
 from .topic import Topic
 
 
-class ClusterTopology(object):
+class ClusterTopology:
     """Represent a Kafka cluster and functionalities supported over the cluster.
 
         :param assignment: cluster assignment is a dict (topic, partition): replicas
@@ -45,19 +43,19 @@ class ClusterTopology(object):
     """
 
     def __init__(
-            self,
-            assignment,
-            brokers,
-            partition_measurer,
-            extract_group=lambda x: None,
-    ):
+        self,
+        assignment: dict[tuple[str, int], list[int]],
+        brokers: dict[int, dict[str, Any] | None],
+        partition_measurer: PartitionMeasurer,
+        extract_group: Callable[[Broker], str],
+    ) -> None:
         self.extract_group = extract_group
         self.partition_measurer = partition_measurer
         self.log = logging.getLogger(self.__class__.__name__)
-        self.topics = {}
-        self.rgs = {}
-        self.brokers = {}
-        self.partitions = {}
+        self.topics: dict[str, Topic] = {}
+        self.rgs: dict[str, ReplicationGroup] = {}
+        self.brokers: dict[int, Broker] = {}
+        self.partitions: dict[tuple[str, int], Partition] = {}
         self._build_brokers(brokers)
         self._build_partitions(assignment)
         self.log.debug(
@@ -76,12 +74,12 @@ class ClusterTopology(object):
             ),
         )
 
-    def _build_brokers(self, brokers):
+    def _build_brokers(self, brokers: dict[int, dict[str, Any] | None]) -> None:
         """Build broker objects using broker-ids."""
-        for broker_id, metadata in six.iteritems(brokers):
+        for broker_id, metadata in brokers.items():
             self.brokers[broker_id] = self._create_broker(broker_id, metadata)
 
-    def _create_broker(self, broker_id, metadata=None):
+    def _create_broker(self, broker_id: int, metadata: dict[str, Any] | None = None) -> Broker:
         """Create a broker object and assign to a replication group.
         A broker object with no metadata is considered inactive.
         An inactive broker may or may not belong to a group.
@@ -95,12 +93,12 @@ class ClusterTopology(object):
         broker.replication_group = group
         return broker
 
-    def _build_partitions(self, assignment):
+    def _build_partitions(self, assignment: dict[tuple[str, int], list[int]]) -> None:
         """Builds all partition objects and update corresponding broker and
         topic objects.
         """
         self.partitions = {}
-        for partition_name, replica_ids in six.iteritems(assignment):
+        for partition_name, replica_ids in assignment.items():
             # Get topic
             topic_id = partition_name[0]
             partition_id = partition_name[1]
@@ -134,48 +132,52 @@ class ClusterTopology(object):
                 self.brokers[broker_id].add_partition(partition)
 
     @property
-    def active_brokers(self):
+    def active_brokers(self) -> set[Broker]:
         """Set of brokers that are not inactive or decommissioned."""
         return {
-            broker for broker in six.itervalues(self.brokers)
+            broker for broker in self.brokers.values()
             if not broker.inactive and not broker.decommissioned
         }
 
     @property
-    def assignment(self):
+    def assignment(self) -> dict[tuple[str, int], list[int]]:
         assignment = {}
-        for partition in six.itervalues(self.partitions):
+        for partition in self.partitions.values():
             assignment[
                 (partition.topic.id, partition.partition_id)
             ] = [broker.id for broker in partition.replicas]
         # assignment map created in sorted order for deterministic solution
         return OrderedDict(sorted(list(assignment.items()), key=lambda t: t[0]))
 
-    def replace_broker(self, source_id, dest_id):
+    def replace_broker(self, source_id: int, dest_id: int) -> None:
         """Move all partitions in source broker to destination broker.
 
         :param source_id: source broker-id
-        :param dest_id: destination broker-id
+        :param dest_id: destination broker-id, or None (to remove source broker)
         :raises: InvalidBrokerIdError, when either of given broker-ids is invalid.
         """
         try:
             source = self.brokers[source_id]
-            dest = self.brokers[dest_id]
+            dest = None
+            if dest_id is not None:
+                dest = self.brokers[dest_id]
             # Move all partitions from source to destination broker
             for partition in source.partitions.copy():  # Partitions set changes
                 # We cannot move partition directly since that re-orders the
                 # replicas for the partition
                 source.partitions.remove(partition)
-                dest.partitions.add(partition)
+                if dest:
+                    dest.partitions.add(partition)
                 # Replace broker in replica
                 partition.replace(source, dest)
+
         except KeyError as e:
             self.log.error("Invalid broker id %s.", e.args[0])
             raise InvalidBrokerIdError(
-                "Broker id {} does not exist in cluster".format(e.args[0])
+                f"Broker id {e.args[0]} does not exist in cluster"
             )
 
-    def update_cluster_topology(self, assignment):
+    def update_cluster_topology(self, assignment: dict[tuple[str, int], list[int]]) -> None:
         """Modify the cluster-topology with given assignment.
 
         Change the replica set of partitions as in given assignment.
@@ -186,7 +188,7 @@ class ClusterTopology(object):
         :raises: InvalidPartitionError when partition-name is invalid
         """
         try:
-            for partition_name, replica_ids in six.iteritems(assignment):
+            for partition_name, replica_ids in assignment.items():
                 try:
                     new_replicas = [self.brokers[b_id] for b_id in replica_ids]
                 except KeyError:
@@ -197,7 +199,7 @@ class ClusterTopology(object):
                         partition_name[1],
                     )
                     raise InvalidBrokerIdError(
-                        "Invalid replicas {0}.".format(
+                        "Invalid replicas {}.".format(
                             ', '.join([str(id) for id in replica_ids])
                         ),
                     )
@@ -225,9 +227,9 @@ class ClusterTopology(object):
                         partition_name[1],
                     )
                     raise InvalidPartitionError(
-                        "Invalid topic-partition {0}-{1}."
+                        "Invalid topic-partition {}-{}."
                         .format(partition_name[0], partition_name[1]),
                     )
         except KeyError:
-            self.log.error("Could not parse given assignment {0}".format(assignment))
+            self.log.error(f"Could not parse given assignment {assignment}")
             raise
