@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 # Copyright 2016 Yelp Inc.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -12,24 +11,21 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
-from __future__ import absolute_import
-from __future__ import division
-from __future__ import print_function
+from __future__ import annotations
 
 import logging
 from collections import OrderedDict
-
-import six
-from six.moves import zip
+from typing import Any
 
 import kafka_utils.kafka_cluster_manager.cluster_info.stats as stats
+from kafka_utils.kafka_cluster_manager.cluster_info.broker import Broker
 from kafka_utils.kafka_cluster_manager.cluster_info.cluster_topology \
     import ClusterTopology
 from kafka_utils.util.validation import assignment_to_plan
 _log = logging.getLogger('kafka-cluster-manager')
 
 
-def display_table(headers, table):
+def display_table(headers: list[str], table: list[list[str]]) -> None:
     """Print a formatted table.
 
     :param headers: A list of header objects that are displayed in the first
@@ -63,7 +59,7 @@ def display_table(headers, table):
         )
 
 
-def _display_table_title_multicolumn(title, key_name, keys, names, values):
+def _display_table_title_multicolumn(title: str, key_name: str, keys: list[Any], names: list[Any], values: list[Any]) -> None:
     assert len(names) == len(values)
     if len(names) == 1:
         headers = [key_name, title]
@@ -73,7 +69,7 @@ def _display_table_title_multicolumn(title, key_name, keys, names, values):
     display_table(headers, list(zip(keys, *values)))
 
 
-def display_replica_imbalance(cluster_topologies):
+def display_replica_imbalance(cluster_topologies: dict[str, ClusterTopology]) -> None:
     """Display replica replication-group distribution imbalance statistics.
 
     :param cluster_topologies: A dictionary mapping a string name to a
@@ -81,10 +77,10 @@ def display_replica_imbalance(cluster_topologies):
     """
     assert cluster_topologies
 
-    rg_ids = list(next(six.itervalues(cluster_topologies)).rgs.keys())
+    rg_ids = list(next(iter(cluster_topologies.values())).rgs.keys())
     assert all(
         set(rg_ids) == set(cluster_topology.rgs.keys())
-        for cluster_topology in six.itervalues(cluster_topologies)
+        for cluster_topology in cluster_topologies.values()
     )
 
     rg_imbalances = [
@@ -92,7 +88,7 @@ def display_replica_imbalance(cluster_topologies):
             list(cluster_topology.rgs.values()),
             list(cluster_topology.partitions.values()),
         )
-        for cluster_topology in six.itervalues(cluster_topologies)
+        for cluster_topology in cluster_topologies.values()
     ]
 
     _display_table_title_multicolumn(
@@ -107,7 +103,7 @@ def display_replica_imbalance(cluster_topologies):
     )
 
     for name, imbalance in zip(
-            six.iterkeys(cluster_topologies),
+            cluster_topologies.keys(),
             (imbalance for imbalance, _ in rg_imbalances)
     ):
         print(
@@ -121,30 +117,30 @@ def display_replica_imbalance(cluster_topologies):
         )
 
 
-def display_partition_imbalance(cluster_topologies):
+def display_partition_imbalance(cluster_topologies: dict[str, ClusterTopology]) -> None:
     """Display partition count and weight imbalance statistics.
 
     :param cluster_topologies: A dictionary mapping a string name to a
         ClusterTopology object.
     """
-    broker_ids = list(next(six.itervalues(cluster_topologies)).brokers.keys())
+    broker_ids = list(next(iter(cluster_topologies.values())).brokers.keys())
     assert all(
         set(broker_ids) == set(cluster_topology.brokers.keys())
-        for cluster_topology in six.itervalues(cluster_topologies)
+        for cluster_topology in cluster_topologies.values()
     )
     broker_partition_counts = [
         stats.get_broker_partition_counts(
             cluster_topology.brokers[broker_id]
             for broker_id in broker_ids
         )
-        for cluster_topology in six.itervalues(cluster_topologies)
+        for cluster_topology in cluster_topologies.values()
     ]
     broker_weights = [
         stats.get_broker_weights(
             cluster_topology.brokers[broker_id]
             for broker_id in broker_ids
         )
-        for cluster_topology in six.itervalues(cluster_topologies)
+        for cluster_topology in cluster_topologies.values()
     ]
 
     _display_table_title_multicolumn(
@@ -187,16 +183,16 @@ def display_partition_imbalance(cluster_topologies):
         )
 
 
-def display_leader_imbalance(cluster_topologies):
+def display_leader_imbalance(cluster_topologies: dict[str, ClusterTopology]) -> None:
     """Display leader count and weight imbalance statistics.
 
     :param cluster_topologies: A dictionary mapping a string name to a
         ClusterTopology object.
     """
-    broker_ids = list(next(six.itervalues(cluster_topologies)).brokers.keys())
+    broker_ids = list(next(iter(cluster_topologies.values())).brokers.keys())
     assert all(
         set(broker_ids) == set(cluster_topology.brokers.keys())
-        for cluster_topology in six.itervalues(cluster_topologies)
+        for cluster_topology in cluster_topologies.values()
     )
 
     broker_leader_counts = [
@@ -204,14 +200,14 @@ def display_leader_imbalance(cluster_topologies):
             cluster_topology.brokers[broker_id]
             for broker_id in broker_ids
         )
-        for cluster_topology in six.itervalues(cluster_topologies)
+        for cluster_topology in cluster_topologies.values()
     ]
     broker_leader_weights = [
         stats.get_broker_leader_weights(
             cluster_topology.brokers[broker_id]
             for broker_id in broker_ids
         )
-        for cluster_topology in six.itervalues(cluster_topologies)
+        for cluster_topology in cluster_topologies.values()
     ]
 
     _display_table_title_multicolumn(
@@ -254,21 +250,21 @@ def display_leader_imbalance(cluster_topologies):
         )
 
 
-def display_topic_broker_imbalance(cluster_topologies):
+def display_topic_broker_imbalance(cluster_topologies: dict[str, ClusterTopology]) -> None:
     """Display topic broker imbalance statistics.
 
     :param cluster_topologies: A dictionary mapping a string name to a
         ClusterTopology object.
     """
-    broker_ids = list(next(six.itervalues(cluster_topologies)).brokers.keys())
+    broker_ids = list(next(iter(cluster_topologies.values())).brokers.keys())
     assert all(
         set(broker_ids) == set(cluster_topology.brokers.keys())
-        for cluster_topology in six.itervalues(cluster_topologies)
+        for cluster_topology in cluster_topologies.values()
     )
-    topic_names = list(next(six.itervalues(cluster_topologies)).topics.keys())
+    topic_names = list(next(iter(cluster_topologies.values())).topics.keys())
     assert all(
         set(topic_names) == set(cluster_topology.topics.keys())
-        for cluster_topology in six.itervalues(cluster_topologies)
+        for cluster_topology in cluster_topologies.values()
     )
 
     imbalances = [
@@ -276,14 +272,14 @@ def display_topic_broker_imbalance(cluster_topologies):
             [cluster_topology.brokers[broker_id] for broker_id in broker_ids],
             [cluster_topology.topics[tname] for tname in topic_names],
         )
-        for cluster_topology in six.itervalues(cluster_topologies)
+        for cluster_topology in cluster_topologies.values()
     ]
     weighted_imbalances = [
         stats.get_weighted_topic_imbalance_stats(
             [cluster_topology.brokers[broker_id] for broker_id in broker_ids],
             [cluster_topology.topics[tname] for tname in topic_names],
         )
-        for cluster_topology in six.itervalues(cluster_topologies)
+        for cluster_topology in cluster_topologies.values()
     ]
 
     _display_table_title_multicolumn(
@@ -311,7 +307,7 @@ def display_topic_broker_imbalance(cluster_topologies):
     )
 
     for name, topic_imbalance, weighted_topic_imbalance in zip(
-            six.iterkeys(cluster_topologies),
+            cluster_topologies.keys(),
             (i[0] for i in imbalances),
             (wi[0] for wi in weighted_imbalances),
     ):
@@ -328,7 +324,7 @@ def display_topic_broker_imbalance(cluster_topologies):
         )
 
 
-def display_movements_stats(ct, base_assignment):
+def display_movements_stats(ct: ClusterTopology, base_assignment: dict[tuple[str, int], list[int]]) -> None:
     """Display how the amount of movement between two assignments.
 
     :param ct: The cluster's ClusterTopology.
@@ -348,13 +344,18 @@ def display_movements_stats(ct, base_assignment):
     )
 
 
-def display_cluster_topology_stats(cluster_topology, base_assignment=None):
+def display_cluster_topology_stats(cluster_topology: ClusterTopology, base_assignment: dict[tuple[str, int], list[int]] | None = None) -> None:
     if base_assignment:
+        def get_replication_group_id(broker: Broker) -> str:
+            replication_group = cluster_topology.brokers[broker.id].replication_group
+            assert replication_group is not None
+            return replication_group.id
+
         base_cluster_topology = ClusterTopology(
             base_assignment,
             {broker.id: broker.metadata for broker in cluster_topology.brokers.values()},
             cluster_topology.partition_measurer,
-            lambda broker: cluster_topology.brokers[broker.id].replication_group.id,
+            get_replication_group_id,
         )
         cluster_topologies = OrderedDict([
             ('Before', base_cluster_topology),
@@ -377,19 +378,19 @@ def display_cluster_topology_stats(cluster_topology, base_assignment=None):
         display_movements_stats(cluster_topology, base_assignment)
 
 
-def display_cluster_topology(cluster_topology):
+def display_cluster_topology(cluster_topology: ClusterTopology) -> None:
     print(assignment_to_plan(cluster_topology.assignment))
 
 
-def display_assignment_changes(plan_details, to_log=True):
+def display_assignment_changes(plan_details: tuple[list[tuple[str, str]], list[tuple[str, str]], int], to_log: bool = True) -> None:
     """Display current and proposed changes in
     topic-partition to replica layout over brokers.
     """
     curr_plan_list, new_plan_list, total_changes = plan_details
-    action_cnt = '\n[INFO] Total actions required {0}'.format(total_changes)
+    action_cnt = f'\n[INFO] Total actions required {total_changes}'
     _log_or_display(to_log, action_cnt)
     action_cnt = (
-        '[INFO] Total actions that will be executed {0}'
+        '[INFO] Total actions that will be executed {}'
         .format(len(new_plan_list))
     )
     _log_or_display(to_log, action_cnt)
@@ -424,7 +425,7 @@ def display_assignment_changes(plan_details, to_log=True):
         proposed_repl = [
             tp_repl[1] for tp_repl in new_plan_list if tp_repl[0] == tp
         ][0]
-        tp_str = '{topic} - {partition:<2d}'.format(topic=tp[0], partition=tp[1])
+        tp_str = f'{tp[0]} - {tp[1]:<2d}'
         row = (
             '{tp:<30s}: {curr_repl:<20s} ==> {proposed_repl:<20s}'.format(
                 tp=tp_str,
@@ -435,7 +436,7 @@ def display_assignment_changes(plan_details, to_log=True):
         _log_or_display(to_log, row)
 
 
-def _log_or_display(to_log, msg):
+def _log_or_display(to_log: bool, msg: str) -> None:
     """Log or display the information."""
     if to_log:
         _log.info(msg)
