@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 # Copyright 2016 Yelp Inc.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -12,17 +11,16 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
-from __future__ import absolute_import
+from __future__ import annotations
 
 import argparse
 import logging
 import sys
+from typing import Any
 
-import six
-from six.moves import filter
-from six.moves import range
-
+from .broker import Broker
 from .cluster_balancer import ClusterBalancer
+from .cluster_topology import ClusterTopology
 from .error import BrokerDecommissionError
 from .error import EmptyReplicationGroupError
 from .error import InvalidBrokerIdError
@@ -30,6 +28,8 @@ from .error import InvalidPartitionError
 from .error import InvalidReplicationFactorError
 from .error import NotEligibleGroupError
 from .error import RebalanceError
+from .partition import Partition
+from .rg import ReplicationGroup
 from .util import compute_optimum
 from .util import separate_groups
 
@@ -43,15 +43,15 @@ class PartitionCountBalancer(ClusterBalancer):
     :param args: The program arguments.
     """
 
-    def __init__(self, cluster_topology, args):
-        super(PartitionCountBalancer, self).__init__(cluster_topology, args)
+    def __init__(self, cluster_topology: ClusterTopology, args: argparse.Namespace) -> None:
+        super().__init__(cluster_topology, args)
         self.log = logging.getLogger(self.__class__.__name__)
 
-    def _set_arg_default(self, arg, value):
+    def _set_arg_default(self, arg: str, value: Any) -> None:
         if not hasattr(self.args, arg):
             setattr(self.args, arg, value)
 
-    def parse_args(self, balancer_args):
+    def parse_args(self, balancer_args: list[str] | None) -> None:
         self._set_arg_default('replication_groups', False)
         self._set_arg_default('brokers', False)
         self._set_arg_default('leaders', False)
@@ -65,7 +65,7 @@ class PartitionCountBalancer(ClusterBalancer):
         )
         parser.parse_args(balancer_args, self.args)
 
-    def decommission_brokers(self, broker_ids):
+    def decommission_brokers(self, broker_ids: list[int]) -> None:
         """Decommission a list of brokers trying to keep the replication group
         the brokers belong to balanced.
 
@@ -81,15 +81,16 @@ class PartitionCountBalancer(ClusterBalancer):
                 # Raise an error for now. As alternative we may ignore the
                 # invalid id and continue with the others.
                 raise InvalidBrokerIdError(
-                    "Broker id {} does not exist in cluster".format(b_id),
+                    f"Broker id {b_id} does not exist in cluster",
                 )
             broker.mark_decommissioned()
+            assert broker.replication_group is not None
             groups.add(broker.replication_group)
 
         for group in groups:
             self._decommission_brokers_in_group(group)
 
-    def _decommission_brokers_in_group(self, group):
+    def _decommission_brokers_in_group(self, group: ReplicationGroup) -> None:
         """Decommission the marked brokers of a group."""
         try:
             group.rebalance_brokers()
@@ -119,9 +120,9 @@ class PartitionCountBalancer(ClusterBalancer):
                     )
                     raise BrokerDecommissionError("Broker decommission failed.")
 
-    def _force_broker_decommission(self, broker):
+    def _force_broker_decommission(self, broker: Broker) -> None:
         available_groups = [
-            rg for rg in six.itervalues(self.cluster_topology.rgs)
+            rg for rg in self.cluster_topology.rgs.values()
             if rg is not broker.replication_group
         ]
 
@@ -144,7 +145,8 @@ class PartitionCountBalancer(ClusterBalancer):
                 except NotEligibleGroupError:
                     pass
 
-    def rebalance(self):
+    def rebalance(self) -> None:
+        assert self.args is not None
         if self.args.max_movement_size:
             self.log.error(
                 '--max-movement-size can not be specified for {balancer}.'
@@ -175,7 +177,7 @@ class PartitionCountBalancer(ClusterBalancer):
             )
             self.rebalance_leaders()
 
-    def rebalance_replication_groups(self):
+    def rebalance_replication_groups(self) -> None:
         """Rebalance partitions over replication groups.
 
         First step involves rebalancing replica-count for each partition across
@@ -184,7 +186,7 @@ class PartitionCountBalancer(ClusterBalancer):
         of the cluster.
         """
         # Balance replicas over replication-groups for each partition
-        if any(b.inactive for b in six.itervalues(self.cluster_topology.brokers)):
+        if any(b.inactive for b in self.cluster_topology.brokers.values()):
             self.log.error(
                 "Impossible to rebalance replication groups because of inactive "
                 "brokers."
@@ -201,12 +203,12 @@ class PartitionCountBalancer(ClusterBalancer):
         self._rebalance_groups_partition_cnt()
 
     # Re-balancing partition count across brokers
-    def rebalance_brokers(self):
+    def rebalance_brokers(self) -> None:
         """Rebalance partition-count across brokers within each replication-group."""
-        for rg in six.itervalues(self.cluster_topology.rgs):
+        for rg in self.cluster_topology.rgs.values():
             rg.rebalance_brokers()
 
-    def revoke_leadership(self, broker_ids):
+    def revoke_leadership(self, broker_ids: list[int]) -> None:
         """Revoke leadership for given brokers.
 
         :param broker_ids: List of broker-ids whose leadership needs to be revoked.
@@ -217,11 +219,11 @@ class PartitionCountBalancer(ClusterBalancer):
             except KeyError:
                 self.log.error("Invalid broker id %s.", b_id)
                 raise InvalidBrokerIdError(
-                    "Broker id {} does not exist in cluster".format(b_id),
+                    f"Broker id {b_id} does not exist in cluster",
                 )
             broker.mark_revoked_leadership()
 
-        assert(len(self.cluster_topology.brokers) - len(broker_ids) > 0), "Not " \
+        assert len(self.cluster_topology.brokers) - len(broker_ids) > 0, "Not " \
             "all brokers can be revoked for leadership"
         opt_leader_cnt = len(self.cluster_topology.partitions) // (
             len(self.cluster_topology.brokers) - len(broker_ids)
@@ -232,13 +234,13 @@ class PartitionCountBalancer(ClusterBalancer):
         # If the broker-ids to be revoked from leadership are still leaders for any
         # partitions, try to forcefully move their leadership to followers if possible
         pending_brokers = [
-            b for b in six.itervalues(self.cluster_topology.brokers)
+            b for b in self.cluster_topology.brokers.values()
             if b.revoked_leadership and b.count_preferred_replica() > 0
         ]
         for b in pending_brokers:
             self._force_revoke_leadership(b)
 
-    def _force_revoke_leadership(self, broker):
+    def _force_revoke_leadership(self, broker: Broker) -> None:
         """Revoke the leadership of given broker for any remaining partitions.
 
         Algorithm:
@@ -281,7 +283,7 @@ class PartitionCountBalancer(ClusterBalancer):
                 )
 
     # Re-balancing leaders
-    def rebalance_leaders(self):
+    def rebalance_leaders(self) -> None:
         """Re-order brokers in replicas such that, every broker is assigned as
         preferred leader evenly.
         """
@@ -289,7 +291,7 @@ class PartitionCountBalancer(ClusterBalancer):
         # Balanced brokers transfer leadership to their under-balanced followers
         self.rebalancing_non_followers(opt_leader_cnt)
 
-    def rebalancing_non_followers(self, opt_cnt):
+    def rebalancing_non_followers(self, opt_cnt: int) -> None:
         """Transfer leadership to any under-balanced followers on the pretext
         that they remain leader-balanced or can be recursively balanced through
         non-followers (followers of other leaders).
@@ -313,27 +315,29 @@ class PartitionCountBalancer(ClusterBalancer):
         # Don't include leaders if they are marked for leadership removal
         under_brokers = list(filter(
             lambda b: b.count_preferred_replica() < opt_cnt and not b.revoked_leadership,
-            six.itervalues(self.cluster_topology.brokers),
+            self.cluster_topology.brokers.values(),
         ))
         if under_brokers:
-            skip_brokers, skip_partitions = [], []
+            skip_brokers: list[Broker] = []
+            skip_partitions: list[Partition] = []
             for broker in under_brokers:
                 skip_brokers.append(broker)
                 broker.request_leadership(opt_cnt, skip_brokers, skip_partitions)
 
         over_brokers = list(filter(
             lambda b: b.count_preferred_replica() > opt_cnt + 1,
-            six.itervalues(self.cluster_topology.brokers),
+            self.cluster_topology.brokers.values(),
         ))
         # Any over-balanced brokers tries to donate their leadership to followers
         if over_brokers:
-            skip_brokers, used_edges = [], []
+            skip_brokers = []
+            used_edges: list[tuple[Partition, Broker, Broker]] = []
             for broker in over_brokers:
                 skip_brokers.append(broker)
                 broker.donate_leadership(opt_cnt, skip_brokers, used_edges)
 
     # Re-balancing partition count across brokers
-    def _rebalance_groups_partition_cnt(self):
+    def _rebalance_groups_partition_cnt(self) -> None:
         """Re-balance partition-count across replication-groups.
 
         Algorithm:
@@ -354,7 +358,7 @@ class PartitionCountBalancer(ClusterBalancer):
         6) Repeat steps 1) to 5) until groups are balanced or cannot be balanced further.
         """
         # Segregate replication-groups based on partition-count
-        total_elements = sum(len(rg.partitions) for rg in six.itervalues(self.cluster_topology.rgs))
+        total_elements = sum(len(rg.partitions) for rg in self.cluster_topology.rgs.values())
         over_loaded_rgs, under_loaded_rgs = separate_groups(
             list(self.cluster_topology.rgs.values()),
             lambda rg: len(rg.partitions),
@@ -411,7 +415,7 @@ class PartitionCountBalancer(ClusterBalancer):
                     # Move to next over-loaded replication-group if balanced
                     break
 
-    def add_replica(self, partition_name, count=1):
+    def add_replica(self, partition_name: tuple[str, int], count: int = 1) -> None:
         """Increase the replication-factor for a partition.
 
         The replication-group to add to is determined as follows:
@@ -431,12 +435,12 @@ class PartitionCountBalancer(ClusterBalancer):
             partition = self.cluster_topology.partitions[partition_name]
         except KeyError:
             raise InvalidPartitionError(
-                "Partition name {name} not found".format(name=partition_name),
+                f"Partition name {partition_name} not found",
             )
         if partition.replication_factor + count > len(self.cluster_topology.brokers):
             raise InvalidReplicationFactorError(
-                "Cannot increase replication factor to {0}. There are only "
-                "{1} brokers."
+                "Cannot increase replication factor to {}. There are only "
+                "{} brokers."
                 .format(
                     partition.replication_factor + count,
                     len(self.cluster_topology.brokers),
@@ -470,7 +474,7 @@ class PartitionCountBalancer(ClusterBalancer):
             if rg.count_replica(partition) >= len(rg.brokers):
                 non_full_rgs.remove(rg)
 
-    def remove_replica(self, partition_name, osr_broker_ids, count=1):
+    def remove_replica(self, partition_name: tuple[str, int], osr_broker_ids: set[int], count: int = 1) -> None:
         """Remove one replica of a partition from the cluster.
 
         The replication-group to remove from is determined as follows:
@@ -498,11 +502,11 @@ class PartitionCountBalancer(ClusterBalancer):
             partition = self.cluster_topology.partitions[partition_name]
         except KeyError:
             raise InvalidPartitionError(
-                "Partition name {name} not found".format(name=partition_name),
+                f"Partition name {partition_name} not found",
             )
         if partition.replication_factor <= count:
             raise InvalidReplicationFactorError(
-                "Cannot remove {0} replicas. Replication factor is only {1}."
+                "Cannot remove {} replicas. Replication factor is only {}."
                 .format(count, partition.replication_factor)
             )
 
@@ -512,7 +516,7 @@ class PartitionCountBalancer(ClusterBalancer):
                 osr.append(self.cluster_topology.brokers[broker_id])
             except KeyError:
                 raise InvalidBrokerIdError(
-                    "No broker found with id {bid}".format(bid=broker_id),
+                    f"No broker found with id {broker_id}",
                 )
 
         non_empty_rgs = [
