@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 # Copyright 2016 Yelp Inc.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -12,23 +11,22 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
-from __future__ import absolute_import
-
 import json
 import logging
 import sys
 from collections import defaultdict
 
-import six
-from six.moves import input
+import humanfriendly
 
 from kafka_utils.kafka_cluster_manager. \
     cluster_info.cluster_topology import ClusterTopology
 from kafka_utils.util.validation import assignment_to_plan
 from kafka_utils.util.zookeeper import ZK
 
+DEFAULT_MAX_MOVEMENT_SIZE = float('inf')
 
-class ClusterManagerCmd(object):
+
+class ClusterManagerCmd:
     """Interface used by all kafka_cluster_manager commands
     The attributes cluster_config, args and zk are initialized on run().
     """
@@ -101,10 +99,10 @@ class ClusterManagerCmd(object):
     def add_subparser(self, subparsers):
         self.build_subparser(subparsers).set_defaults(command=self.run)
 
-    def execute_plan(self, plan, allow_rf_change=False):
+    def execute_plan(self, plan, allow_rf_change=False, allow_rf_mismatch=False):
         """Save proposed-plan and execute the same if requested."""
         if self.should_execute():
-            result = self.zk.execute_plan(plan, allow_rf_change=allow_rf_change)
+            result = self.zk.execute_plan(plan, allow_rf_change=allow_rf_change, allow_rf_mismatch=allow_rf_mismatch)
             if not result:
                 self.log.error('Plan execution unsuccessful.')
                 sys.exit(1)
@@ -136,7 +134,7 @@ class ClusterManagerCmd(object):
         else:
             return False
 
-    def process_assignment(self, assignment, allow_rf_change=False):
+    def process_assignment(self, assignment, allow_rf_change=False, allow_rf_mismatch=False):
         plan = assignment_to_plan(assignment)
         if self.args.proposed_plan_file:
             self.log.info(
@@ -152,14 +150,16 @@ class ClusterManagerCmd(object):
             'Proposed-plan actions count: %s',
             len(plan['partitions']),
         )
-        self.execute_plan(plan, allow_rf_change=allow_rf_change)
+        self.execute_plan(plan, allow_rf_change=allow_rf_change, allow_rf_mismatch=allow_rf_mismatch)
 
     def get_reduced_assignment(
         self,
         original_assignment,
-        new_assignment,
+        cluster_topology,
         max_partition_movements,
         max_leader_only_changes,
+        max_movement_size=DEFAULT_MAX_MOVEMENT_SIZE,
+        force_progress=False,
     ):
         """Reduce the assignment based on the total actions.
 
@@ -170,21 +170,25 @@ class ClusterManagerCmd(object):
 
         Argument(s):
         original_assignment:    Current assignment of cluster in zookeeper
-        new_assignment:         New proposed-assignment of cluster
+        cluster_topology:       Cluster topology containing the new proposed-assignment of cluster
         max_partition_movements:Maximum number of partition-movements in
                                 final set of actions
         max_leader_only_changes:Maximum number of actions with leader only changes
+        max_movement_size:      Maximum size, in bytes, to move in final set of actions
+        force_progress:         Whether to force progress if max_movement_size is too small
         :return:
         :reduced_assignment:    Final reduced assignment
         """
+        new_assignment = cluster_topology.assignment
         if (not original_assignment or not new_assignment or
-                max_partition_movements < 0 or max_leader_only_changes < 0):
+                max_partition_movements < 0 or max_leader_only_changes < 0 or
+                max_movement_size < 0):
             return {}
 
         # The replica set stays the same for leaders only changes
         leaders_changes = [
             (t_p, new_assignment[t_p])
-            for t_p, replica in six.iteritems(original_assignment)
+            for t_p, replica in original_assignment.items()
             if replica != new_assignment[t_p] and
             set(replica) == set(new_assignment[t_p])
         ]
@@ -196,7 +200,7 @@ class ClusterManagerCmd(object):
                 t_p,
                 len(set(replica) - set(new_assignment[t_p])),
             )
-            for t_p, replica in six.iteritems(original_assignment)
+            for t_p, replica in original_assignment.items()
             if set(replica) != set(new_assignment[t_p])
         ]
 
@@ -204,11 +208,33 @@ class ClusterManagerCmd(object):
             "Total number of actions before reduction: %s.",
             len(partition_change_count) + len(leaders_changes),
         )
-        # Extract reduced plan maximizing uniqueness of topics
+        # Extract reduced plan maximizing uniqueness of topics and ensuring we do not
+        # go over the max_movement_size
         reduced_actions = self._extract_actions_unique_topics(
             partition_change_count,
             max_partition_movements,
+            cluster_topology,
+            max_movement_size,
         )
+
+        # Ensure progress is made if force_progress=True
+        if len(reduced_actions) == 0 and force_progress:
+            smallest_size = min([cluster_topology.partitions[t_p[0]].size for t_p in partition_change_count])
+            self.log.warning(
+                '--max-movement-size={max_movement_size} is too small, using smallest size'
+                ' in set of partitions to move, {smallest_size} instead to force progress'.format(
+                    max_movement_size=humanfriendly.format_size(max_movement_size),
+                    smallest_size=humanfriendly.format_size(smallest_size),
+                )
+            )
+            max_movement_size = smallest_size
+            reduced_actions = self._extract_actions_unique_topics(
+                partition_change_count,
+                max_partition_movements,
+                cluster_topology,
+                max_movement_size,
+            )
+
         reduced_partition_changes = [
             (t_p, new_assignment[t_p]) for t_p in reduced_actions
         ]
@@ -227,7 +253,7 @@ class ClusterManagerCmd(object):
         }
         return reduced_assignment
 
-    def _extract_actions_unique_topics(self, movement_counts, max_movements):
+    def _extract_actions_unique_topics(self, movement_counts, max_movements, cluster_topology, max_movement_size):
         """Extract actions limiting to given max value such that
            the resultant has the minimum possible number of duplicate topics.
 
@@ -238,6 +264,8 @@ class ClusterManagerCmd(object):
               are reached.
            :param movement_counts: list of tuple ((topic, partition), movement count)
            :param max_movements: max number of movements to extract
+           :param cluster_topology: cluster topology containing the new proposed assignment for the cluster
+           :param max_movement_size: maximum size of data to move at a time in extracted actions
            :return: list of tuple (topic, partitions) to include in the reduced plan
         """
         # Group actions by topic
@@ -248,19 +276,24 @@ class ClusterManagerCmd(object):
         # Create reduced assignment minimizing duplication of topics
         extracted_actions = []
         curr_movements = 0
+        curr_size = 0
         action_available = True
-        while curr_movements < max_movements and action_available:
+        while curr_movements < max_movements and curr_size <= max_movement_size and action_available:
             action_available = False
-            for topic, actions in six.iteritems(topic_actions):
-                for action in actions:
-                    if curr_movements + action[1] > max_movements:
-                        # Remove action since it won't be possible to use it
+            for topic, actions in topic_actions.items():
+                # Iterate through a copy of actions so we can remove elements in the original list
+                for action in actions[:]:
+                    action_size = cluster_topology.partitions[action[0]].size
+                    if curr_movements + action[1] > max_movements or curr_size + action_size > max_movement_size:
+                        # Remove this action from pool of actions since we can't use it
                         actions.remove(action)
                     else:
                         # Append (topic, partition) to the list of movements
                         action_available = True
                         extracted_actions.append(action[0])
                         curr_movements += action[1]
+                        curr_size += action_size
+                        # Remove this action from pool of actions since we have used it
                         actions.remove(action)
                         break
         return extracted_actions
@@ -279,3 +312,19 @@ class ClusterManagerCmd(object):
         """Dump proposed json plan to given output file for future usage."""
         with open(proposed_plan_file, 'w') as output:
             json.dump(proposed_layout, output)
+
+    # For use in commands that accept topic filters, e.g. replace-broker and preferred-replica-election
+    def get_topic_filter(self):
+        if self.args.topic_partition_filter is None:
+            self.log.error('Need to specify topic partition filter file')
+            sys.exit(1)
+        filter_set = set()
+        with open(self.args.topic_partition_filter) as f:
+            for line in f:
+                tokens = line.split(':')
+                if len(tokens) != 2:
+                    self.log.error("Invalid topic partition filter line: %s", line)
+                    sys.exit(1)
+                t_p = (tokens[0].strip(), int(tokens[1].strip()))
+                filter_set.add(t_p)
+        return filter_set
