@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 # Copyright 2016 Yelp Inc.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -12,17 +11,12 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
-from __future__ import absolute_import
-from __future__ import print_function
-from __future__ import unicode_literals
-
 import argparse
+import configparser
 import logging
 import sys
 from logging.config import fileConfig
 
-import six.moves.configparser
-
 from kafka_utils.kafka_cluster_manager.cluster_info.cluster_balancer \
     import ClusterBalancer
 from kafka_utils.kafka_cluster_manager.cluster_info.partition_count_balancer \
@@ -36,6 +30,7 @@ from kafka_utils.kafka_cluster_manager.cluster_info.replication_group_parser \
 from kafka_utils.kafka_cluster_manager.cluster_info.replication_group_parser \
     import ReplicationGroupParser
 from kafka_utils.kafka_cluster_manager.cmds.decommission import DecommissionCmd
+from kafka_utils.kafka_cluster_manager.cmds.preferred_replica_election import PreferredReplicaElectionCmd
 from kafka_utils.kafka_cluster_manager.cmds.rebalance import RebalanceCmd
 from kafka_utils.kafka_cluster_manager.cmds.replace import ReplaceBrokerCmd
 from kafka_utils.kafka_cluster_manager.cmds.revoke_leadership import RevokeLeadershipCmd
@@ -170,6 +165,7 @@ def parse_args():
     StoreAssignmentsCmd().add_subparser(subparsers)
     ReplaceBrokerCmd().add_subparser(subparsers)
     SetReplicationFactorCmd().add_subparser(subparsers)
+    PreferredReplicaElectionCmd().add_subparser(subparsers)
 
     return parser.parse_args()
 
@@ -188,7 +184,7 @@ def configure_logging(log_conf=None, log_unhandled_exceptions=True):
     if log_conf:
         try:
             fileConfig(log_conf, disable_existing_loggers=False)
-        except six.moves.configparser.NoSectionError:
+        except configparser.NoSectionError:
             logging.basicConfig(level=logging.INFO)
             _log.error(
                 'Failed to load {logconf} file.'
