@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 # Copyright 2016 Yelp Inc.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -12,14 +11,15 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
-from __future__ import absolute_import
-from __future__ import print_function
-from __future__ import unicode_literals
+from __future__ import annotations
 
+import argparse
 import sys
+from typing import Any
 
 from .offset_manager import OffsetManagerBase
 from kafka_utils.util.client import KafkaToolClient
+from kafka_utils.util.config import ClusterConfig
 from kafka_utils.util.offsets import get_current_consumer_offsets
 from kafka_utils.util.offsets import set_consumer_offsets
 
@@ -27,7 +27,7 @@ from kafka_utils.util.offsets import set_consumer_offsets
 class CopyGroup(OffsetManagerBase):
 
     @classmethod
-    def setup_subparser(cls, subparsers):
+    def setup_subparser(cls, subparsers: Any) -> None:
         parser_copy_group = subparsers.add_parser(
             "copy_group",
             description="Copy specified consumer group details to a new group.",
@@ -56,7 +56,7 @@ class CopyGroup(OffsetManagerBase):
         parser_copy_group.set_defaults(command=cls.run)
 
     @classmethod
-    def run(cls, args, cluster_config):
+    def run(cls, args: argparse.Namespace, cluster_config: ClusterConfig) -> None:
         if args.source_groupid == args.dest_groupid:
             print(
                 "Error: Source group ID and destination group ID are same.",
@@ -72,6 +72,7 @@ class CopyGroup(OffsetManagerBase):
             args.partitions,
             cluster_config,
             client,
+            use_admin_client=args.use_admin_client,
         )
 
         cls.copy_group_kafka(
@@ -82,6 +83,6 @@ class CopyGroup(OffsetManagerBase):
         )
 
     @classmethod
-    def copy_group_kafka(cls, client, topics, source_group, destination_group):
+    def copy_group_kafka(cls, client: KafkaToolClient, topics: dict[str, list[int]], source_group: str, destination_group: str) -> None:
         copied_offsets = get_current_consumer_offsets(client, source_group, topics)
         set_consumer_offsets(client, destination_group, copied_offsets)
