@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 # Copyright 2016 Yelp Inc.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -12,11 +11,14 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
-from __future__ import absolute_import
-from __future__ import print_function
+from __future__ import annotations
+
+import argparse
+from typing import Any
 
 from .offset_manager import OffsetWriter
 from kafka_utils.util.client import KafkaToolClient
+from kafka_utils.util.config import ClusterConfig
 from kafka_utils.util.offsets import nullify_offsets
 from kafka_utils.util.offsets import set_consumer_offsets
 
@@ -24,7 +26,7 @@ from kafka_utils.util.offsets import set_consumer_offsets
 class DeleteGroup(OffsetWriter):
 
     @classmethod
-    def setup_subparser(cls, subparsers):
+    def setup_subparser(cls, subparsers: Any):
         parser_delete_group = subparsers.add_parser(
             "delete_group",
             description="Delete a consumer group by groupid. This "
@@ -42,7 +44,7 @@ class DeleteGroup(OffsetWriter):
         parser_delete_group.set_defaults(command=cls.run)
 
     @classmethod
-    def run(cls, args, cluster_config):
+    def run(cls, args: argparse.Namespace, cluster_config: ClusterConfig) -> None:
         # Setup the Kafka client
         client = KafkaToolClient(cluster_config.broker_list)
         client.load_metadata_for_topics()
@@ -53,10 +55,11 @@ class DeleteGroup(OffsetWriter):
             None,
             cluster_config,
             client,
+            use_admin_client=args.use_admin_client,
         )
         cls.delete_group_kafka(client, args.groupid, topics_dict)
 
     @classmethod
-    def delete_group_kafka(cls, client, group, topics):
+    def delete_group_kafka(cls, client: KafkaToolClient, group: str, topics: dict[str, dict[int, int]]) -> None:
         new_offsets = nullify_offsets(topics)
         set_consumer_offsets(client, group, new_offsets)
