@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 # Copyright 2016 Yelp Inc.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -12,21 +11,19 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
-from __future__ import absolute_import
-from __future__ import print_function
-
+import argparse
 import sys
-
-import six
+from typing import Any
 
 from .offset_manager import OffsetManagerBase
 from kafka_utils.util.client import KafkaToolClient
+from kafka_utils.util.config import ClusterConfig
 
 
 class ListTopics(OffsetManagerBase):
 
     @classmethod
-    def setup_subparser(cls, subparsers):
+    def setup_subparser(cls, subparsers: Any) -> None:
         parser_list_topics = subparsers.add_parser(
             "list_topics",
             description="List topics by consumer group.",
@@ -43,7 +40,7 @@ class ListTopics(OffsetManagerBase):
         parser_list_topics.set_defaults(command=cls.run)
 
     @classmethod
-    def run(cls, args, cluster_config):
+    def run(cls, args: argparse.Namespace, cluster_config: ClusterConfig) -> None:
         # Setup the Kafka client
         client = KafkaToolClient(cluster_config.broker_list)
         client.load_metadata_for_topics()
@@ -55,6 +52,7 @@ class ListTopics(OffsetManagerBase):
             cluster_config=cluster_config,
             client=client,
             fail_on_error=False,
+            use_admin_client=args.use_admin_client,
         )
         if not topics_dict:
             print("Consumer Group ID: {group} does not exist.".format(
@@ -62,7 +60,7 @@ class ListTopics(OffsetManagerBase):
             ))
             sys.exit(1)
 
-        print("Consumer Group ID: {groupid}".format(groupid=args.groupid))
-        for topic, partitions in six.iteritems(topics_dict):
-            print("\tTopic: {topic}".format(topic=topic))
-            print("\t\tPartitions: {partitions}".format(partitions=partitions))
+        print(f"Consumer Group ID: {args.groupid}")
+        for topic, partitions in topics_dict.items():
+            print(f"\tTopic: {topic}")
+            print(f"\t\tPartitions: {partitions}")
