@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 # Copyright 2016 Yelp Inc.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -12,21 +11,22 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
-from __future__ import absolute_import
-from __future__ import print_function
-from __future__ import unicode_literals
-
+import argparse
 import sys
+from typing import Any
+
+from kafka.errors import UnknownMemberIdError
 
 from .offset_manager import OffsetWriter
 from kafka_utils.util.client import KafkaToolClient
+from kafka_utils.util.config import ClusterConfig
 from kafka_utils.util.offsets import advance_consumer_offsets
 
 
 class OffsetAdvance(OffsetWriter):
 
     @classmethod
-    def setup_subparser(cls, subparsers):
+    def setup_subparser(cls, subparsers: Any) -> None:
         parser_offset_advance = subparsers.add_parser(
             "offset_advance",
             description="Advance consumer offsets for the specified consumer "
@@ -62,7 +62,7 @@ class OffsetAdvance(OffsetWriter):
         parser_offset_advance.set_defaults(command=cls.run)
 
     @classmethod
-    def run(cls, args, cluster_config):
+    def run(cls, args: argparse.Namespace, cluster_config: ClusterConfig) -> None:
         # Setup the Kafka client
         client = KafkaToolClient(cluster_config.broker_list)
         client.load_metadata_for_topics()
@@ -74,6 +74,7 @@ class OffsetAdvance(OffsetWriter):
             cluster_config,
             client,
             force=args.force,
+            use_admin_client=args.use_admin_client,
         )
         try:
             advance_consumer_offsets(
@@ -87,5 +88,13 @@ class OffsetAdvance(OffsetWriter):
                 "with --help option.", file=sys.stderr
             )
             raise
+        except UnknownMemberIdError:
+            print(
+                "Unable to advance offsets for group '{group_name}' from topic '{topic_name}'. \
+                    You must ensure none of the consumers with this consumer group id are running before \
+                    trying to advance the offsets stored in Kafka for this consumer group. Try stopping all \
+                    of your consumers.".format(group_name=args.groupid, topic_name=args.topic),
+            )
+            raise
 
         client.close()
