@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 # Copyright 2016 Yelp Inc.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -12,29 +11,30 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
-from __future__ import absolute_import
-from __future__ import print_function
-from __future__ import unicode_literals
+from __future__ import annotations
 
 import sys
 
-import six
 from kazoo.exceptions import NoNodeError
 
-from kafka_utils.kafka_consumer_manager.util import KafkaGroupReader
+from kafka_utils.kafka_consumer_manager.util import get_kafka_group_reader
 from kafka_utils.kafka_consumer_manager.util import prompt_user_input
+from kafka_utils.util.config import ClusterConfig
 from kafka_utils.util.zookeeper import ZK
 
 
-class OffsetManagerBase(object):
+class OffsetManagerBase:
 
     @classmethod
     def get_topics_from_consumer_group_id(
         cls,
-        cluster_config,
-        groupid,
+        cluster_config: ClusterConfig,
+        groupid: int,
+        use_admin_client: bool = False,
     ):
-        return cls.get_topics_for_group_from_kafka(cluster_config, groupid)
+        return cls.get_topics_for_group_from_kafka(
+            cluster_config, groupid, use_admin_client,
+        )
 
     @classmethod
     def preprocess_args(
@@ -47,6 +47,7 @@ class OffsetManagerBase(object):
         fail_on_error=True,
         quiet=False,
         topics=None,
+        use_admin_client=False,
     ):
         if topics and partitions:
             print(
@@ -80,6 +81,7 @@ class OffsetManagerBase(object):
         subscribed_topics = cls.get_topics_from_consumer_group_id(
             cluster_config,
             groupid,
+            use_admin_client,
         )
         topics_dict = {}
 
@@ -140,18 +142,30 @@ class OffsetManagerBase(object):
     @classmethod
     def get_topics_for_group_from_kafka(
         cls,
-        cluster_config,
-        groupid,
+        cluster_config: ClusterConfig,
+        groupid: int,
+        use_admin_client: bool = False,
     ):
-        kafka_group_reader = KafkaGroupReader(cluster_config)
-        group_topics = kafka_group_reader.read_group(groupid)
-        if len(group_topics) == 0:
+        kafka_group_reader = get_kafka_group_reader(
+            cluster_config, use_admin_client,
+        )
+        try:
+            group_topics = kafka_group_reader.read_group(groupid)
+        except KeyError:
             print(
-                "Error: Either Consumer Group {groupid} has never committed offsets, or the group does not exist.".format(
+                "Warning: Consumer Group {groupid} does not exist.".format(
                     groupid=groupid,
                 ),
                 file=sys.stderr,
             )
+            group_topics = []
+        if len(group_topics) == 0:
+            print(
+                "Info: It's the first time Consumer Group {groupid} is committing offsets.".format(
+                    groupid=groupid,
+                ),
+                file=sys.stdout,
+            )
         return group_topics
 
     @classmethod
@@ -200,8 +214,9 @@ class OffsetWriter(OffsetManagerBase):
         fail_on_error=True,
         force=False,
         topics=None,
+        use_admin_client=False,
     ):
-        topics_dict = super(OffsetWriter, cls).preprocess_args(
+        topics_dict = super().preprocess_args(
             groupid,
             topic,
             partitions,
@@ -209,6 +224,7 @@ class OffsetWriter(OffsetManagerBase):
             client,
             fail_on_error=(fail_on_error and not force),
             topics=topics,
+            use_admin_client=use_admin_client,
         )
 
         if not topics_dict and force:
@@ -220,7 +236,7 @@ class OffsetWriter(OffsetManagerBase):
             )
 
         topics_str = ""
-        for local_topic, local_partitions in six.iteritems(topics_dict):
+        for local_topic, local_partitions in topics_dict.items():
             temp_str = "Topic: {topic}, Partitions: {partitions}\n".format(
                 topic=local_topic,
                 partitions=local_partitions
@@ -239,7 +255,7 @@ class OffsetWriter(OffsetManagerBase):
 
     @classmethod
     def get_forced_topic_partitions(cls, groupid, topic, partitions, client):
-        assert(topic is not None)
+        assert topic is not None
         if not partitions:
             partitions = client.get_partition_ids_for_topic(topic)
         return {topic: partitions}
