@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 # Copyright 2016 Yelp Inc.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -12,10 +11,6 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
-from __future__ import absolute_import
-from __future__ import print_function
-from __future__ import unicode_literals
-
 import sys
 
 from .offset_manager import OffsetWriter
@@ -74,6 +69,7 @@ class OffsetRewind(OffsetWriter):
             cluster_config,
             client,
             force=args.force,
+            use_admin_client=args.use_admin_client,
         )
         try:
             rewind_consumer_offsets(
