@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 # Copyright 2016 Yelp Inc.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -12,12 +11,11 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
-from __future__ import absolute_import
-from __future__ import print_function
-from __future__ import unicode_literals
+from __future__ import annotations
 
 import sys
 from collections import defaultdict
+from typing import DefaultDict
 
 from .offset_manager import OffsetWriter
 from kafka_utils.util.client import KafkaToolClient
@@ -25,7 +23,7 @@ from kafka_utils.util.offsets import set_consumer_offsets
 
 
 class OffsetSet(OffsetWriter):
-    new_offsets_dict = defaultdict(dict)
+    new_offsets_dict: DefaultDict[str, dict[int, int]] = defaultdict(dict)
 
     @classmethod
     def topics_dict(cls, string):
