@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 # Copyright 2016 Yelp Inc.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -12,10 +11,6 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
-from __future__ import absolute_import
-from __future__ import print_function
-from __future__ import unicode_literals
-
 import sys
 
 from .offset_manager import OffsetManagerBase
@@ -67,6 +62,7 @@ class RenameGroup(OffsetManagerBase):
             partitions=None,
             cluster_config=cluster_config,
             client=client,
+            use_admin_client=args.use_admin_client,
         )
         cls.rename_group(
             client,
