@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 # Copyright 2016 Yelp Inc.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -12,10 +11,9 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
-from __future__ import absolute_import
-from __future__ import print_function
+import sys
 
-import six
+from kafka.errors import UnknownMemberIdError
 
 from .offset_manager import OffsetWriter
 from kafka_utils.util.client import KafkaToolClient
@@ -77,6 +75,7 @@ class UnsubscribeTopics(OffsetWriter):
             cluster_config,
             client,
             topics=args.topics,
+            use_admin_client=args.use_admin_client,
         )
 
         topics = args.topics if args.topics else ([args.topic] if args.topic else [])
@@ -90,7 +89,7 @@ class UnsubscribeTopics(OffsetWriter):
         )
 
 
-class TopicUnsubscriber(object):
+class TopicUnsubscriber:
     """Base class used to unsubscribe consumer groups from topic partitions."""
 
     def unsubscribe_topics(self, group, topics, partitions, topics_dict):
@@ -107,9 +106,20 @@ class TopicUnsubscriber(object):
             self.unsubscribe_partitions(group, topics[0], partitions)
         elif topics:
             for topic in topics:
-                self.delete_topic(group, topic)
+                try:
+                    self.delete_topic(group, topic)
+                except UnknownMemberIdError:
+                    print(
+                        "Error: Unable to unsubscribe group '{group_name}' from topic '{topic_name}'. \
+                            Please ensure all consumers in this consumer group are stopped before \
+                            trying to unsubscribe a consumer group with offsets stored in \
+                            Kafka.".format(group_name=group, topic_name=topic),
+                        file=sys.stderr,
+                    )
+                    raise
+
         else:
-            for topic, partitions in six.iteritems(topics_dict):
+            for topic, partitions in topics_dict.items():
                 self.delete_topic(group, topic)
 
     def unsubscribe_partitions(self, group, topic, partitions):
