@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 # Copyright 2016 Yelp Inc.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -12,10 +11,6 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
-from __future__ import absolute_import
-from __future__ import print_function
-from __future__ import unicode_literals
-
 import argparse
 import logging
 import sys
@@ -80,6 +75,11 @@ def parse_args():
         action='store_true',
         help='Show DEBUG level logging',
     )
+    parser.add_argument(
+        "--use-admin-client", action="store_true",
+        help="Use the Kafka AdminClient interface to fetch consumer "
+        "group offsets for quicker performance."
+    )
     subparsers = parser.add_subparsers()
 
     OffsetGet.add_parser(subparsers)
