@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 # Copyright 2016 Yelp Inc.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -12,22 +11,19 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
-from __future__ import absolute_import
-from __future__ import print_function
-from __future__ import unicode_literals
+from __future__ import annotations
 
 import logging
 import sys
 from collections import defaultdict
 
-import six
+from kafka.admin import KafkaAdminClient
 from kafka.consumer import KafkaConsumer
 from kafka.structs import OffsetAndMetadata
 from kafka.structs import TopicPartition
 from kafka.util import read_short_string
 from kafka.util import relative_unpack
 from kazoo.exceptions import NodeExistsError
-from six.moves import input
 
 from kafka_utils.util.client import KafkaToolClient
 from kafka_utils.util.error import UnknownTopic
@@ -82,8 +78,8 @@ def create_offsets(zk, consumer_group, offsets):
     :type offsets: dict(topic, dict(partition, offset))
     """
     # Create new offsets
-    for topic, partition_offsets in six.iteritems(offsets):
-        for partition, offset in six.iteritems(partition_offsets):
+    for topic, partition_offsets in offsets.items():
+        for partition, offset in partition_offsets.items():
             new_path = "/consumers/{groupid}/offsets/{topic}/{partition}".format(
                 groupid=consumer_group,
                 topic=topic,
@@ -109,7 +105,7 @@ def fetch_offsets(zk, consumer_group, topics):
     :rtype: dict(topic, dict(partition, offset))
     """
     source_offsets = defaultdict(dict)
-    for topic, partitions in six.iteritems(topics):
+    for topic, partitions in topics.items():
         for partition in partitions:
             offset, _ = zk.get(
                 "/consumers/{groupid}/offsets/{topic}/{partition}".format(
@@ -123,7 +119,7 @@ def fetch_offsets(zk, consumer_group, topics):
 
 
 def prompt_user_input(in_str):
-    while(True):
+    while True:
         answer = input(in_str + ' ')
         if answer == "n" or answer == "no":
             sys.exit(0)
@@ -211,7 +207,7 @@ def consumer_partitions_for_topic(consumer, topic):
             topic_partitions.append(TopicPartition(topic, partition))
     else:
         logging.error(
-            "No partitions found for topic {}. Maybe it doesn't exist?".format(topic),
+            f"No partitions found for topic {topic}. Maybe it doesn't exist?",
         )
     return topic_partitions
 
@@ -227,7 +223,7 @@ def consumer_commit_for_times(consumer, partition_to_offset, atomic=False):
             TopicPartition: timestamp pairs.
     """
     no_offsets = set()
-    for tp, offset in six.iteritems(partition_to_offset):
+    for tp, offset in partition_to_offset.items():
         if offset is None:
             logging.error(
                 "No offsets found for topic-partition {tp}. Either timestamps not supported"
@@ -244,7 +240,7 @@ def consumer_commit_for_times(consumer, partition_to_offset, atomic=False):
 
     offsets_metadata = {
         tp: OffsetAndMetadata(partition_to_offset[tp].offset, metadata=None)
-        for tp in six.iterkeys(partition_to_offset) if tp not in no_offsets
+        for tp in partition_to_offset.keys() if tp not in no_offsets
     }
 
     if len(offsets_metadata) != 0:
@@ -255,6 +251,44 @@ class InvalidMessageException(Exception):
     pass
 
 
+class KafkaAdminGroupReader:
+
+    def __init__(self, kafka_config):
+        self.log = logging.getLogger(__name__)
+        self.admin_client = KafkaAdminClient(
+            bootstrap_servers=kafka_config.broker_list,
+        )
+
+    def read_group(self, groupid: int) -> list[str]:
+        topics = set()
+        group_offsets = self.admin_client.list_consumer_group_offsets(groupid)
+        for tp in group_offsets.keys():
+            topics.add(tp.topic)
+
+        return list(topics)
+
+    def read_groups(self, groupids: list[int] | None = None, list_only: bool = False) -> dict[int, list[str]]:
+        if groupids is None:
+            groupids = self._list_groups()
+
+        if list_only:
+            return {groupid: [] for groupid in groupids}
+
+        groups = {}
+        for groupid in groupids:
+            topics = self.read_group(groupid)
+            groups[groupid] = topics
+
+        return groups
+
+    def _list_groups(self) -> list[int]:
+        groups_and_protocol_types = self.admin_client.list_consumer_groups()
+        return [
+            gpt[0]
+            for gpt in groups_and_protocol_types
+        ]
+
+
 class KafkaGroupReader:
 
     def __init__(self, kafka_config):
@@ -267,9 +301,9 @@ class KafkaGroupReader:
     def read_group(self, group_id):
         partition_count = get_offset_topic_partition_count(self.kafka_config)
         partition = get_group_partition(group_id, partition_count)
-        return self.read_groups(partition).get(group_id, [])
+        return self.read_groups(partition)[group_id]
 
-    def read_groups(self, partition=None):
+    def read_groups(self, partition=None, list_only: bool = False) -> dict[int, list[str]]:
         self.consumer = KafkaConsumer(
             group_id='offset_monitoring_consumer',
             bootstrap_servers=self.kafka_config.broker_list,
@@ -280,6 +314,10 @@ class KafkaGroupReader:
             max_partition_fetch_bytes=10 * 1024 * 1024,  # 10MB
         )
 
+        # Fetch metadata as partitions_for_topic only returns locally cached metadata
+        # See https://github.com/dpkp/kafka-python/issues/1742
+        self.consumer.topics()
+
         if partition is not None:
             self.active_partitions = {
                 partition: TopicPartition(CONSUMER_OFFSET_TOPIC, partition),
@@ -321,17 +359,17 @@ class KafkaGroupReader:
 
         return {
             group: topics.keys()
-            for group, topics in six.iteritems(self._kafka_groups)
+            for group, topics in self._kafka_groups.items()
             if topics
         }
 
     def _remove_unsubscribed_topics(self):
-        for group, topics in list(six.iteritems(self._kafka_groups)):
-            for topic, partitions in list(six.iteritems(topics)):
+        for group, topics in list(self._kafka_groups.items()):
+            for topic, partitions in list(topics.items()):
                 # If offsets for all partitions are 0, consider the topic as unsubscribed
                 if not any(partitions.values()):
                     del self._kafka_groups[group][topic]
-                    self.log.info("Removed group {group} topic {topic} from list of groups".format(group=group, topic=topic))
+                    self.log.info(f"Removed group {group} topic {topic} from list of groups")
 
     def remove_partition_from_consumer(self, partition):
         deleted = self.active_partitions.pop(partition)
@@ -355,7 +393,7 @@ class KafkaGroupReader:
             self.active_partitions,
         )
 
-    def parse_consumer_offset_message(self, message):
+    def parse_consumer_offset_message(self, message) -> tuple[str, str, int, int]:
         key = message.key
         ((key_schema,), cur) = relative_unpack(b'>h', key, 0)
         if key_schema not in [0, 1]:
@@ -366,7 +404,7 @@ class KafkaGroupReader:
         if message.value:
             value = message.value
             ((value_schema,), cur) = relative_unpack(b'>h', value, 0)
-            if value_schema not in [0, 1]:
+            if value_schema not in [0, 1, 3]:
                 raise InvalidMessageException()  # Unrecognized message value
             ((offset,), cur) = relative_unpack(b'>q', value, cur)
         else:
@@ -391,7 +429,7 @@ class KafkaGroupReader:
         elif offset is None and group in self._kafka_groups and \
                 topic in self._kafka_groups[group]:  # No offset means topic deletion
             del self._kafka_groups[group][topic]
-            self.log.info("Removed group {group} topic {topic} from list of groups".format(group=group, topic=topic))
+            self.log.info(f"Removed group {group} topic {topic} from list of groups")
 
     def get_current_watermarks(self, partitions=None):
         client = KafkaToolClient(self.kafka_config.broker_list)
@@ -400,11 +438,18 @@ class KafkaGroupReader:
             client,
             [CONSUMER_OFFSET_TOPIC],
         )
-        partitions_set = set(tp.partition for tp in partitions) if partitions else None
+        partitions_set = {tp.partition for tp in partitions} if partitions else None
         return {part: offset for part, offset
-                in six.iteritems(offsets[CONSUMER_OFFSET_TOPIC])
+                in offsets[CONSUMER_OFFSET_TOPIC].items()
                 if offset.highmark > offset.lowmark and
                 (partitions is None or part in partitions_set)}
 
     def finished(self):
         return self._finished
+
+
+def get_kafka_group_reader(cluster_config, use_admin_client=False) -> KafkaGroupReader | KafkaAdminGroupReader:
+    if use_admin_client:
+        return KafkaAdminGroupReader(cluster_config)
+    else:
+        return KafkaGroupReader(cluster_config)
