@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 # Copyright 2016 Yelp Inc.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -12,10 +11,6 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
-from __future__ import absolute_import
-from __future__ import division
-from __future__ import print_function
-
 import argparse
 import itertools
 import logging
@@ -26,8 +21,6 @@ from operator import itemgetter
 
 from requests.exceptions import RequestException
 from requests_futures.sessions import FuturesSession
-from six.moves import input
-from six.moves import zip
 
 from .task import PostStopTask
 from .task import PreStopTask
@@ -47,6 +40,8 @@ DEFAULT_CHECK_COUNT = 12
 DEFAULT_TIME_LIMIT_SECS = 600
 DEFAULT_JOLOKIA_PORT = 8778
 DEFAULT_JOLOKIA_PREFIX = "jolokia/"
+DEFAULT_JOLOKIA_USER = None
+DEFAULT_JOLOKIA_PASSWORD = None
 DEFAULT_STOP_COMMAND = "service kafka stop"
 DEFAULT_START_COMMAND = "service kafka start"
 
@@ -117,6 +112,18 @@ def parse_opts():
         type=str,
         default=DEFAULT_JOLOKIA_PREFIX,
     )
+    parser.add_argument(
+        '--jolokia-user',
+        help='the jolokia username. Default: %(default)s',
+        type=str,
+        default=DEFAULT_JOLOKIA_USER,
+    )
+    parser.add_argument(
+        '--jolokia-password',
+        help='the jolokia password. Default: %(default)s',
+        type=str,
+        default=DEFAULT_JOLOKIA_PASSWORD,
+    )
     parser.add_argument(
         '--no-confirm',
         help='proceed without asking confirmation. Default: %(default)s',
@@ -129,6 +136,11 @@ def parse_opts():
         type=int,
         default=0,
     )
+    parser.add_argument(
+        '--active-controller-for-last',
+        help=('leave the active controller for last. Default: %(default)s'),
+        action='store_true'
+    )
     parser.add_argument(
         '-v',
         '--verbose',
@@ -171,14 +183,25 @@ def parse_opts():
     return parser.parse_args()
 
 
-def get_broker_list(cluster_config):
-    """Returns a list of brokers in the form [(id: host)]
+def get_broker_list(cluster_config, active_controller_for_last=False):
+    """Returns a dictionary of brokers in the form {id: host}
 
     :param cluster_config: the configuration of the cluster
     :type cluster_config: map
+    :param active_controller_for_last: end with active controller
+    :type active_controller_for_last: bool
     """
     with ZK(cluster_config) as zk:
-        brokers = sorted(list(zk.get_brokers().items()), key=itemgetter(0))
+        brokers_dict = zk.get_brokers()
+        if active_controller_for_last:
+            active_controller_id = zk.get_json('/controller').get('brokerid')
+            active_controller = brokers_dict.pop(active_controller_id, None)
+            if active_controller is None:
+                print("ERROR: Unable to retrieve the active controller information")
+                sys.exit(1)
+        brokers = sorted(list(brokers_dict.items()), key=itemgetter(0))
+        if active_controller_for_last:
+            brokers.append((active_controller_id, active_controller))
         return [(id, data['host']) for id, data in brokers]
 
 
@@ -195,7 +218,7 @@ def filter_broker_list(brokers, filter_by):
     return [(id, host) for id, host in brokers if id in filter_by_set]
 
 
-def generate_requests(hosts, jolokia_port, jolokia_prefix):
+def generate_requests(hosts, jolokia_port, jolokia_prefix, jolokia_user, jolokia_password):
     """Return a generator of requests to fetch the under replicated
     partition number from the specified hosts.
 
@@ -205,9 +228,15 @@ def generate_requests(hosts, jolokia_port, jolokia_prefix):
     :type jolokia_port: integer
     :param jolokia_prefix: HTTP prefix on the server for the Jolokia queries
     :type jolokia_prefix: string
+    :param jolokia_user: Username for Jolokia, if needed
+    :type jolokia_user: string
+    :param jolokia_password: Password for Jolokia, if needed
+    :type jolokia_password: string
     :returns: generator of requests
     """
     session = FuturesSession()
+    if jolokia_user and jolokia_password:
+        session.auth = (jolokia_user, jolokia_password)
     for host in hosts:
         url = "http://{host}:{port}/{prefix}/read/{key}".format(
             host=host,
@@ -218,7 +247,7 @@ def generate_requests(hosts, jolokia_port, jolokia_prefix):
         yield host, session.get(url)
 
 
-def read_cluster_status(hosts, jolokia_port, jolokia_prefix):
+def read_cluster_status(hosts, jolokia_port, jolokia_prefix, jolokia_user, jolokia_password):
     """Read and return the number of under replicated partitions and
     missing brokers from the specified hosts.
 
@@ -229,19 +258,26 @@ def read_cluster_status(hosts, jolokia_port, jolokia_prefix):
     :param jolokia_prefix: HTTP prefix on the server for the Jolokia queries
     :type jolokia_prefix: string
     :returns: tuple of integers
+    :param jolokia_user: Username for Jolokia, if needed
+    :type jolokia_user: string
+    :param jolokia_password: Password for Jolokia, if needed
+    :type jolokia_password: string
     """
     under_replicated = 0
     missing_brokers = 0
-    for host, request in generate_requests(hosts, jolokia_port, jolokia_prefix):
+    for host, request in generate_requests(hosts, jolokia_port, jolokia_prefix, jolokia_user, jolokia_password):
         try:
             response = request.result()
+            if response.status_code == 401:
+                print("Jolokia Authentication Failed. Exiting.")
+                sys.exit(1)
             if 400 <= response.status_code <= 599:
-                print("Got status code {0}. Exiting.".format(response.status_code))
+                print(f"Got status code {response.status_code}. Exiting.")
                 sys.exit(1)
             json = response.json()
             under_replicated += json['value']
         except RequestException as e:
-            print("Broker {0} is down: {1}."
+            print("Broker {} is down: {}."
                   "This maybe because it is starting up".format(host, e), file=sys.stderr)
             missing_brokers += 1
         except KeyError:
@@ -258,9 +294,9 @@ def print_brokers(cluster_config, brokers):
     :param brokers: the brokers that will be restarted
     :type brokers: map of broker ids and host names
     """
-    print("Will restart the following brokers in {0}:".format(cluster_config.name))
+    print(f"Will restart the following brokers in {cluster_config.name}:")
     for id, host in brokers:
-        print("  {0}: {1}".format(id, host))
+        print(f"  {id}: {host}")
 
 
 def ask_confirmation():
@@ -300,6 +336,8 @@ def wait_for_stable_cluster(
     hosts,
     jolokia_port,
     jolokia_prefix,
+    jolokia_user,
+    jolokia_password,
     check_interval,
     check_count,
     unhealthy_time_limit,
@@ -313,6 +351,10 @@ def wait_for_stable_cluster(
     :type jolokia_port: integer
     :param jolokia_prefix: HTTP prefix on the server for the Jolokia queries
     :type jolokia_prefix: string
+    :param jolokia_user: Username for Jolokia, if needed
+    :type jolokia_user: string
+    :param jolokia_password: Password for Jolokia, if needed
+    :type jolokia_password: string
     :param check_interval: the number of seconds it will wait between each check
     :type check_interval: integer
     :param check_count: the number of times the check should be positive before
@@ -329,6 +371,8 @@ def wait_for_stable_cluster(
             hosts,
             jolokia_port,
             jolokia_prefix,
+            jolokia_user,
+            jolokia_password
         )
         if partitions or brokers:
             stable_counter = 0
@@ -361,6 +405,8 @@ def execute_rolling_restart(
     brokers,
     jolokia_port,
     jolokia_prefix,
+    jolokia_user,
+    jolokia_password,
     check_interval,
     check_count,
     unhealthy_time_limit,
@@ -385,6 +431,10 @@ def execute_rolling_restart(
     :type jolokia_port: integer
     :param jolokia_prefix: HTTP prefix on the server for the Jolokia queries
     :type jolokia_prefix: string
+    :param jolokia_user: Username for Jolokia, if needed
+    :type jolokia_user: string
+    :param jolokia_password: Password for Jolokia, if needed
+    :type jolokia_password: string
     :param check_interval: the number of seconds it will wait between each check
     :type check_interval: integer
     :param check_count: the number of times the check should be positive before
@@ -417,23 +467,27 @@ def execute_rolling_restart(
                 all_hosts,
                 jolokia_port,
                 jolokia_prefix,
+                jolokia_user,
+                jolokia_password,
                 check_interval,
                 1 if n == 0 else check_count,
                 unhealthy_time_limit,
             )
-            print("Stopping {0} ({1}/{2})".format(host, n + 1, len(all_hosts) - skip))
+            print(f"Stopping {host} ({n + 1}/{len(all_hosts) - skip})")
             stop_broker(host, connection, stop_command, verbose)
             execute_task(post_stop_task, host)
         # we open a new SSH connection in case the hostname has a new IP
         with ssh(host=host, forward_agent=True, sudoable=True, max_attempts=3, max_timeout=2,
                  ssh_password=ssh_password) as connection:
-            print("Starting {0} ({1}/{2})".format(host, n + 1, len(all_hosts) - skip))
+            print(f"Starting {host} ({n + 1}/{len(all_hosts) - skip})")
             start_broker(host, connection, start_command, verbose)
     # Wait before terminating the script
     wait_for_stable_cluster(
         all_hosts,
         jolokia_port,
         jolokia_prefix,
+        jolokia_user,
+        jolokia_password,
         check_interval,
         check_count,
         unhealthy_time_limit,
@@ -482,7 +536,7 @@ def validate_broker_ids_subset(broker_ids, subset_ids):
     for subset_id in subset_ids:
         valid = valid and subset_id in all_ids
         if subset_id not in all_ids:
-            print("Error: user specified broker id {0} does not exist in cluster.".format(subset_id))
+            print(f"Error: user specified broker id {subset_id} does not exist in cluster.")
     return valid
 
 
@@ -525,7 +579,7 @@ def run():
         opts.cluster_name,
         opts.discovery_base_path,
     )
-    brokers = get_broker_list(cluster_config)
+    brokers = get_broker_list(cluster_config, opts.active_controller_for_last)
     if opts.broker_ids:
         if not validate_broker_ids_subset([id for id, host in brokers], opts.broker_ids):
             sys.exit(1)
@@ -544,6 +598,8 @@ def run():
                 brokers,
                 opts.jolokia_port,
                 opts.jolokia_prefix,
+                opts.jolokia_user,
+                opts.jolokia_password,
                 opts.check_interval,
                 opts.check_count,
                 opts.unhealthy_time_limit,
