@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 # Copyright 2016 Yelp Inc.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -12,16 +11,16 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
-from __future__ import absolute_import
+from __future__ import annotations
 
 import glob
 import logging
 import os
-from collections import namedtuple
+from typing import Iterator
+from typing import NamedTuple
 
-import six
 import yaml
-from six.moves import map
+from typing_extensions import TypedDict
 
 from kafka_utils.util.error import ConfigurationError
 from kafka_utils.util.error import InvalidConfigurationError
@@ -32,29 +31,28 @@ DEFAULT_KAFKA_TOPOLOGY_BASE_PATH = '/etc/kafka_discovery'
 HOME_OVERRIDE = '.kafka_discovery'
 
 
-class ClusterConfig(
-    namedtuple(
-        'ClusterConfig',
-        ['type', 'name', 'broker_list', 'zookeeper'],
-    ),
-):
+class ClusterConfig(NamedTuple):
     """Cluster configuration.
     :param name: cluster name
     :param broker_list: list of kafka brokers
     :param zookeeper: zookeeper connection string
     """
+    type: str
+    name: str
+    broker_list: list[str]
+    zookeeper: str
 
-    def __ne__(self, other):
+    def __ne__(self, other: object) -> bool:
         return self.__hash__() != other.__hash__()
 
-    def __eq__(self, other):
+    def __eq__(self, other: object) -> bool:
         return self.__hash__() == other.__hash__()
 
-    def __hash__(self):
+    def __hash__(self) -> int:
         if isinstance(self.broker_list, list):
             broker_list = self.broker_list
         else:
-            broker_list = self.broker_list.split(',')
+            broker_list = self.broker_list.split(',')  # type: ignore[unreachable]
         zk_list = self.zookeeper.split(',')
         return hash((
             self.type,
@@ -64,12 +62,26 @@ class ClusterConfig(
         ))
 
 
-def load_yaml_config(config_path):
-    with open(config_path, 'r') as config_file:
+class ClusterConfigDict(TypedDict):
+    broker_list: list[str]
+    zookeeper: str
+
+
+class LocalConfigDict(TypedDict):
+    cluster: str
+
+
+class TopologyConfigurationDict(TypedDict):
+    clusters: dict[str, ClusterConfigDict]
+    local_config: LocalConfigDict
+
+
+def load_yaml_config(config_path: str) -> TopologyConfigurationDict:
+    with open(config_path) as config_file:
         return yaml.safe_load(config_file)
 
 
-class TopologyConfiguration(object):
+class TopologyConfiguration:
     """Topology configuration for a kafka cluster.
 
     Read a cluster_type.yaml from the kafka_topology_path.
@@ -100,17 +112,18 @@ class TopologyConfiguration(object):
 
     def __init__(
         self,
-        cluster_type,
-        kafka_topology_path=DEFAULT_KAFKA_TOPOLOGY_BASE_PATH
+        cluster_type: str,
+        kafka_topology_path: str = DEFAULT_KAFKA_TOPOLOGY_BASE_PATH,
     ):
         self.kafka_topology_path = kafka_topology_path
         self.cluster_type = cluster_type
         self.log = logging.getLogger(self.__class__.__name__)
-        self.clusters = None
-        self.local_config = None
+        self.clusters: dict[str, ClusterConfigDict] | None = None
+        self.local_config: LocalConfigDict | None = None
         self.load_topology_config()
 
-    def __eq__(self, other):
+    def __eq__(self, other: object) -> bool:
+        assert isinstance(other, TopologyConfiguration)
         if all([
             self.cluster_type == other.cluster_type,
             self.clusters == other.clusters,
@@ -119,21 +132,21 @@ class TopologyConfiguration(object):
             return True
         return False
 
-    def __ne__(self, other):
+    def __ne__(self, other: object) -> bool:
         return not self.__eq__(other)
 
-    def load_topology_config(self):
+    def load_topology_config(self) -> None:
         """Load the topology configuration"""
         config_path = os.path.join(
             self.kafka_topology_path,
-            '{id}.yaml'.format(id=self.cluster_type),
+            f'{self.cluster_type}.yaml',
         )
         self.log.debug("Loading configuration from %s", config_path)
         if os.path.isfile(config_path):
             topology_config = load_yaml_config(config_path)
         else:
             raise MissingConfigurationError(
-                "Topology configuration {0} for cluster {1} "
+                "Topology configuration {} for cluster {} "
                 "does not exist".format(
                     config_path,
                     self.cluster_type,
@@ -144,12 +157,13 @@ class TopologyConfiguration(object):
             self.clusters = topology_config['clusters']
         except KeyError:
             self.log.exception("Invalid topology file")
-            raise InvalidConfigurationError("Invalid topology file {0}".format(
+            raise InvalidConfigurationError("Invalid topology file {}".format(
                 config_path))
         if 'local_config' in topology_config:
             self.local_config = topology_config['local_config']
 
-    def get_all_clusters(self):
+    def get_all_clusters(self) -> list[ClusterConfig]:
+        assert self.clusters is not None
         return [
             ClusterConfig(
                 type=self.cluster_type,
@@ -157,10 +171,11 @@ class TopologyConfiguration(object):
                 broker_list=cluster['broker_list'],
                 zookeeper=cluster['zookeeper'],
             )
-            for name, cluster in six.iteritems(self.clusters)
+            for name, cluster in self.clusters.items()
         ]
 
-    def get_cluster_by_name(self, name):
+    def get_cluster_by_name(self, name: str) -> ClusterConfig:
+        assert self.clusters is not None
         if name in self.clusters:
             cluster = self.clusters[name]
             return ClusterConfig(
@@ -169,11 +184,12 @@ class TopologyConfiguration(object):
                 broker_list=cluster['broker_list'],
                 zookeeper=cluster['zookeeper'],
             )
-        raise ConfigurationError("No cluster with name: {0}".format(name))
+        raise ConfigurationError(f"No cluster with name: {name}")
 
-    def get_local_cluster(self):
+    def get_local_cluster(self) -> ClusterConfig:
         if self.local_config:
             try:
+                assert self.clusters is not None
                 local_cluster = self.clusters[self.local_config['cluster']]
                 return ClusterConfig(
                     type=self.cluster_type,
@@ -186,16 +202,16 @@ class TopologyConfiguration(object):
         else:
             raise ConfigurationError("No default local cluster configured")
 
-    def __repr__(self):
-        return ("TopologyConfig: cluster_type {0}, clusters: {1},"
-                "local_config {2}".format(
+    def __repr__(self) -> str:
+        return ("TopologyConfig: cluster_type {}, clusters: {},"
+                "local_config {}".format(
                     self.cluster_type,
                     self.clusters,
                     self.local_config
                 ))
 
 
-def get_conf_dirs():
+def get_conf_dirs() -> list[str]:
     config_dirs = []
     if os.environ.get("KAFKA_DISCOVERY_DIR"):
         config_dirs.append(os.environ["KAFKA_DISCOVERY_DIR"])
@@ -211,10 +227,10 @@ def get_conf_dirs():
 
 
 def get_cluster_config(
-    cluster_type,
-    cluster_name=None,
-    kafka_topology_base_path=None,
-):
+    cluster_type: str,
+    cluster_name: str | None = None,
+    kafka_topology_base_path: str | None = None,
+) -> ClusterConfig:
     """Return the cluster configuration.
     Use the local cluster if cluster_name is not specified.
 
@@ -243,7 +259,7 @@ def get_cluster_config(
             pass
     if not topology:
         raise MissingConfigurationError(
-            "No available configuration for type {0}".format(cluster_type),
+            f"No available configuration for type {cluster_type}",
         )
 
     if cluster_name:
@@ -252,7 +268,7 @@ def get_cluster_config(
         return topology.get_local_cluster()
 
 
-def iter_configurations(kafka_topology_base_path=None):
+def iter_configurations(kafka_topology_base_path: str | None = None) -> Iterator[TopologyConfiguration]:
     """Cluster topology iterator.
     Iterate over all the topologies available in config.
     """
@@ -265,7 +281,7 @@ def iter_configurations(kafka_topology_base_path=None):
     for config_dir in config_dirs:
         new_types = [x for x in map(
             lambda x: os.path.basename(x)[:-5],
-            glob.glob('{0}/*.yaml'.format(config_dir)),
+            glob.glob(f'{config_dir}/*.yaml'),
         ) if x not in types]
         for cluster_type in new_types:
             try:
