@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 # Copyright 2016 Yelp Inc.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -12,43 +11,57 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
-from __future__ import absolute_import
+from __future__ import annotations
 
 from collections import defaultdict
-from collections import namedtuple
+from typing import cast
+from typing import Collection
+from typing import List
+from typing import Mapping
+from typing import NamedTuple
+from typing import TypeVar
+from typing import Union
 
-import six
 from kafka.common import UnknownTopicOrPartitionError
 from kafka.errors import BrokerResponseError
 from kafka.errors import check_error
 from kafka.structs import OffsetCommitRequestPayload
+from kafka.structs import OffsetCommitResponsePayload
 from kafka.structs import OffsetFetchRequestPayload
 from kafka.structs import OffsetFetchResponsePayload
 from kafka.structs import OffsetRequestPayload
 from kafka.structs import OffsetResponsePayload
+from typing_extensions import Protocol
+from typing_extensions import TypedDict
 
+from kafka_utils.util.client import KafkaToolClient
 from kafka_utils.util.error import OffsetCommitError
 from kafka_utils.util.error import UnknownPartitions
 from kafka_utils.util.error import UnknownTopic
 
 
-PartitionOffsets = namedtuple(
-    'PartitionOffsets',
-    ['topic', 'partition', 'highmark', 'lowmark']
-)
-"""Tuple representing the offsets for a topic partition.
+TopicsCollection = Union[List[str], Mapping[str, Collection[int]]]
+
+
+class PartitionOffsets(NamedTuple):
+    r"""Tuple representing the offsets for a topic partition.
+
+    * **topic**\(``str``): Name of the topic
+    * **partition**\(``int``): Partition number
+    * **highmark**\(``int``): high watermark
+    * **lowmark**\(``int``): low watermark
+    """
+    topic: str
+    partition: int
+    highmark: int
+    lowmark: int
 
-* **topic**\(``str``): Name of the topic
-* **partition**\(``int``): Partition number
-* **highmark**\(``int``): high watermark
-* **lowmark**\(``int``): low watermark
-"""
 
 HIGH_WATERMARK = "high"
 LOW_WATERMARK = "low"
 
 
-def pluck_topic_offset_or_zero_on_unknown(resp):
+def pluck_topic_offset_or_zero_on_unknown(resp: OffsetFetchResponsePayload) -> OffsetFetchResponsePayload:
     try:
         check_error(resp)
     except UnknownTopicOrPartitionError:
@@ -68,7 +81,7 @@ def pluck_topic_offset_or_zero_on_unknown(resp):
     return resp
 
 
-def _check_fetch_response_error(resp):
+def _check_fetch_response_error(resp: OffsetResponsePayload) -> OffsetResponsePayload:
     try:
         check_error(resp)
     except BrokerResponseError:
@@ -82,7 +95,16 @@ def _check_fetch_response_error(resp):
     return resp
 
 
-def _check_commit_response_error(resp):
+class Response(Protocol):
+    topic: str
+    partition: int
+    error: int
+
+
+ResponseT = TypeVar('ResponseT', bound=Response)
+
+
+def _check_commit_response_error(resp: ResponseT) -> ResponseT | OffsetCommitError:
     try:
         check_error(resp)
     except BrokerResponseError as e:
@@ -95,21 +117,22 @@ def _check_commit_response_error(resp):
     return resp
 
 
-def _validate_topics_list_or_dict(topics):
-    if isinstance(topics, dict):
+def _validate_topics_list_or_dict(topics: TopicsCollection) -> Mapping[str, Collection[int]]:
+    if isinstance(topics, Mapping):
         return topics
     elif isinstance(topics, (list, set, tuple)):
-        return dict([(topic, []) for topic in topics])
+        return {topic: [] for topic in topics}
     else:
         raise TypeError("Invalid topics: {topics}. It must be either a "
                         "list of topics or a dict "
                         "topic: [partitions]".format(topics=topics))
 
 
-def _verify_topics_and_partitions(kafka_client, topics, raise_on_error):
+def _verify_topics_and_partitions(kafka_client: KafkaToolClient, topics: TopicsCollection, raise_on_error: bool) -> Mapping[str, Collection[int]]:
     topics = _validate_topics_list_or_dict(topics)
     valid_topics = {}
-    for topic, partitions in six.iteritems(topics):
+    partitions: Collection[int]
+    for topic, partitions in topics.items():
         # Check topic exists
         if not kafka_client.has_metadata_for_topic(topic):
             if raise_on_error:
@@ -140,7 +163,7 @@ def _verify_topics_and_partitions(kafka_client, topics, raise_on_error):
     return valid_topics
 
 
-def _verify_commit_offsets_requests(kafka_client, new_offsets, raise_on_error):
+def _verify_commit_offsets_requests(kafka_client: KafkaToolClient, new_offsets: dict[str, dict[int, int]], raise_on_error: bool) -> dict[str, dict[int, int]]:
     type_error_str = (
         "Invalid new_offsets: {new_offsets}. It must be a "
         "dict of the format: "
@@ -150,33 +173,33 @@ def _verify_commit_offsets_requests(kafka_client, new_offsets, raise_on_error):
     if not isinstance(new_offsets, dict):
         raise TypeError(type_error_str)
 
-    for topic, partitions in six.iteritems(new_offsets):
+    for topic, partitions in new_offsets.items():
         if not isinstance(partitions, dict):
             raise TypeError(type_error_str)
 
-    topics = dict(
-        (topic, list(partitions.keys()))
-        for topic, partitions in six.iteritems(new_offsets)
-    )
+    topics = {
+        topic: list(partitions.keys())
+        for topic, partitions in new_offsets.items()
+    }
 
     valid_topics = _verify_topics_and_partitions(kafka_client, topics, raise_on_error)
 
-    return dict(
-        (topic, dict(
-            (partition, new_offsets[topic][partition])
+    return {
+        topic: {
+            partition: new_offsets[topic][partition]
             for partition in partitions
-        ))
-        for topic, partitions in six.iteritems(valid_topics)
+        }
+        for topic, partitions in valid_topics.items()
         if partitions
-    )
+    }
 
 
 def get_current_consumer_offsets(
-    kafka_client,
-    group,
-    topics,
-    raise_on_error=True,
-):
+    kafka_client: KafkaToolClient,
+    group: str,
+    topics: TopicsCollection,
+    raise_on_error: bool = True,
+) -> dict[str, dict[int, int]]:
     """ Get current consumer offsets.
 
     NOTE: This method does not refresh client metadata. It is up to the caller
@@ -202,15 +225,15 @@ def get_current_consumer_offsets(
       FailedPayloadsError: upon send request error.
     """
 
-    topics = _verify_topics_and_partitions(kafka_client, topics, raise_on_error)
+    topics_dict = _verify_topics_and_partitions(kafka_client, topics, raise_on_error)
 
     group_offset_reqs = [
         OffsetFetchRequestPayload(topic, partition)
-        for topic, partitions in six.iteritems(topics)
+        for topic, partitions in topics_dict.items()
         for partition in partitions
     ]
 
-    group_offsets = {}
+    group_offsets: dict[str, dict[int, int]] = {}
 
     send_api = kafka_client.send_offset_fetch_request_kafka
 
@@ -231,7 +254,12 @@ def get_current_consumer_offsets(
     return group_offsets
 
 
-def get_topics_watermarks(kafka_client, topics, raise_on_error=True):
+class WatermarksDict(TypedDict, total=False):
+    lowmark: int
+    highmark: int
+
+
+def get_topics_watermarks(kafka_client: KafkaToolClient, topics: TopicsCollection, raise_on_error: bool = True) -> dict[str, dict[int, PartitionOffsets]]:
     """ Get current topic watermarks.
 
     NOTE: This method does not refresh client metadata. It is up to the caller
@@ -255,7 +283,7 @@ def get_topics_watermarks(kafka_client, topics, raise_on_error=True):
 
       FailedPayloadsError: upon send request error.
     """
-    topics = _verify_topics_and_partitions(
+    topics_dict = _verify_topics_and_partitions(
         kafka_client,
         topics,
         raise_on_error,
@@ -263,7 +291,7 @@ def get_topics_watermarks(kafka_client, topics, raise_on_error=True):
     highmark_offset_reqs = []
     lowmark_offset_reqs = []
 
-    for topic, partitions in six.iteritems(topics):
+    for topic, partitions in topics_dict.items():
         # Batch watermark requests
         for partition in partitions:
             # Request the the latest offset
@@ -279,7 +307,7 @@ def get_topics_watermarks(kafka_client, topics, raise_on_error=True):
                 )
             )
 
-    watermark_offsets = {}
+    watermark_offsets: dict[str, dict[int, PartitionOffsets]] = {}
 
     if not (len(highmark_offset_reqs) + len(lowmark_offset_reqs)):
         return watermark_offsets
@@ -298,16 +326,14 @@ def get_topics_watermarks(kafka_client, topics, raise_on_error=True):
 
     # At this point highmark and lowmark should ideally have the same length.
     assert len(highmark_resps) == len(lowmark_resps)
-    aggregated_offsets = defaultdict(lambda: defaultdict(dict))
+    aggregated_offsets: dict[str, dict[int, WatermarksDict]] = defaultdict(lambda: defaultdict(lambda: cast(WatermarksDict, {})))
     for resp in highmark_resps:
-        aggregated_offsets[resp.topic][resp.partition]['highmark'] = \
-            resp.offsets[0]
+        aggregated_offsets[resp.topic][resp.partition]['highmark'] = resp.offsets[0]
     for resp in lowmark_resps:
-        aggregated_offsets[resp.topic][resp.partition]['lowmark'] = \
-            resp.offsets[0]
+        aggregated_offsets[resp.topic][resp.partition]['lowmark'] = resp.offsets[0]
 
-    for topic, partition_watermarks in six.iteritems(aggregated_offsets):
-        for partition, watermarks in six.iteritems(partition_watermarks):
+    for topic, partition_watermarks in aggregated_offsets.items():
+        for partition, watermarks in partition_watermarks.items():
             watermark_offsets.setdefault(
                 topic,
                 {},
@@ -321,15 +347,15 @@ def get_topics_watermarks(kafka_client, topics, raise_on_error=True):
 
 
 def _commit_offsets_to_watermark(
-    kafka_client,
-    group,
-    topics,
-    watermark,
-    raise_on_error,
-):
-    topics = _verify_topics_and_partitions(kafka_client, topics, raise_on_error)
+    kafka_client: KafkaToolClient,
+    group: str,
+    topics: TopicsCollection,
+    watermark: str,
+    raise_on_error: bool,
+) -> list[OffsetCommitResponsePayload | OffsetCommitError]:
+    topics_dict = _verify_topics_and_partitions(kafka_client, topics, raise_on_error)
 
-    watermark_offsets = get_topics_watermarks(kafka_client, topics, raise_on_error)
+    watermark_offsets = get_topics_watermarks(kafka_client, topics_dict, raise_on_error)
 
     if watermark == HIGH_WATERMARK:
         group_offset_reqs = [
@@ -338,7 +364,7 @@ def _commit_offsets_to_watermark(
                 watermark_offsets[topic][partition].highmark,
                 metadata=''
             )
-            for topic, partitions in six.iteritems(topics)
+            for topic, partitions in topics_dict.items()
             for partition in partitions
         ]
     elif watermark == LOW_WATERMARK:
@@ -348,12 +374,12 @@ def _commit_offsets_to_watermark(
                 watermark_offsets[topic][partition].lowmark,
                 metadata=''
             )
-            for topic, partitions in six.iteritems(topics)
+            for topic, partitions in topics_dict.items()
             for partition in partitions
         ]
     else:
         raise ValueError(
-            "Unknown watermark: {watermark}".format(watermark=watermark)
+            f"Unknown watermark: {watermark}"
         )
 
     send_api = kafka_client.send_offset_commit_request_kafka
@@ -371,11 +397,11 @@ def _commit_offsets_to_watermark(
 
 
 def advance_consumer_offsets(
-    kafka_client,
-    group,
-    topics,
-    raise_on_error=True,
-):
+    kafka_client: KafkaToolClient,
+    group: str,
+    topics: TopicsCollection,
+    raise_on_error: bool = True,
+) -> list[OffsetCommitResponsePayload | OffsetCommitError]:
     """Advance consumer offsets to the latest message in the topic
     partition (the high watermark).
 
@@ -411,11 +437,11 @@ def advance_consumer_offsets(
 
 
 def rewind_consumer_offsets(
-    kafka_client,
-    group,
-    topics,
-    raise_on_error=True,
-):
+    kafka_client: KafkaToolClient,
+    group: str,
+    topics: TopicsCollection,
+    raise_on_error: bool = True,
+) -> list[OffsetCommitResponsePayload | OffsetCommitError]:
     """Rewind consumer offsets to the earliest message in the topic
     partition (the low watermark).
 
@@ -451,11 +477,11 @@ def rewind_consumer_offsets(
 
 
 def set_consumer_offsets(
-    kafka_client,
-    group,
-    new_offsets,
-    raise_on_error=True,
-):
+    kafka_client: KafkaToolClient,
+    group: str,
+    new_offsets: dict[str, dict[int, int]],
+    raise_on_error: bool = True,
+) -> list[OffsetCommitResponsePayload | OffsetCommitError]:
     """Set consumer offsets to the specified offsets.
 
     This method does not validate the specified offsets, it is up to
@@ -497,8 +523,8 @@ def set_consumer_offsets(
             offset,
             metadata='',
         )
-        for topic, new_partition_offsets in six.iteritems(valid_new_offsets)
-        for partition, offset in six.iteritems(new_partition_offsets)
+        for topic, new_partition_offsets in valid_new_offsets.items()
+        for partition, offset in new_partition_offsets.items()
     ]
 
     send_api = kafka_client.send_offset_commit_request_kafka
@@ -516,14 +542,14 @@ def set_consumer_offsets(
             if _f and _f.error != 0]
 
 
-def _nullify_partition_offsets(partition_offsets):
+def _nullify_partition_offsets(partition_offsets: dict[int, int]) -> dict[int, int]:
     result = {}
     for partition in partition_offsets:
         result[partition] = 0
     return result
 
 
-def nullify_offsets(offsets):
+def nullify_offsets(offsets: dict[str, dict[int, int]]) -> dict[str, dict[int, int]]:
     """Modify offsets metadata so that the partition offsets
     have null payloads.
 
@@ -531,6 +557,6 @@ def nullify_offsets(offsets):
     :returns: a dict topic: partition: offset
     """
     result = {}
-    for topic, partition_offsets in six.iteritems(offsets):
+    for topic, partition_offsets in offsets.items():
         result[topic] = _nullify_partition_offsets(partition_offsets)
     return result
