@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 # Copyright 2016 Yelp Inc.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -13,23 +12,16 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 import json
-
-import six
+from typing import Any
 
 
-def load_json(input_data):
-    if six.PY3:
-        data_str = input_data.decode()
-    else:
-        data_str = input_data
+def load_json(input_data: bytes) -> Any:
+    data_str = input_data.decode()
 
     return json.loads(data_str)
 
 
-def dump_json(obj):
+def dump_json(obj: Any) -> bytes:
     serialized = json.dumps(obj, sort_keys=True)
 
-    if six.PY3:
-        serialized = serialized.encode()
-
-    return serialized
+    return serialized.encode()
