@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 # Copyright 2016 Yelp Inc.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -13,17 +12,29 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 """"Provide functions to validate and generate a Kafka assignment"""
-from __future__ import absolute_import
+from __future__ import annotations
 
 import logging
 from collections import Counter
 
-import six
+from typing_extensions import TypedDict
+
 
 _log = logging.getLogger(__name__)
 
 
-def plan_to_assignment(plan):
+class PartitionDict(TypedDict):
+    topic: str
+    partition: int
+    replicas: list[int]
+
+
+class PlanDict(TypedDict):
+    version: int
+    partitions: list[PartitionDict]
+
+
+def plan_to_assignment(plan: PlanDict) -> dict[tuple[str, int], list[int]]:
     """Convert the plan to the format used by cluster-topology."""
     assignment = {}
     for elem in plan['partitions']:
@@ -33,7 +44,7 @@ def plan_to_assignment(plan):
     return assignment
 
 
-def assignment_to_plan(assignment):
+def assignment_to_plan(assignment: dict[tuple[str, int], list[int]]) -> PlanDict:
     """Convert an assignment to the format used by Kafka to
     describe a reassignment plan.
     """
@@ -43,16 +54,17 @@ def assignment_to_plan(assignment):
         [{'topic': t_p[0],
           'partition': t_p[1],
           'replicas': replica
-          } for t_p, replica in six.iteritems(assignment)]
+          } for t_p, replica in assignment.items()]
     }
 
 
 def validate_plan(
-    new_plan,
-    base_plan=None,
-    is_partition_subset=True,
-    allow_rf_change=False,
-):
+    new_plan: PlanDict,
+    base_plan: PlanDict | None = None,
+    is_partition_subset: bool = True,
+    allow_rf_change: bool = False,
+    allow_rf_mismatch: bool = False,
+) -> bool:
     """Verify that the new plan is valid for execution.
 
     Given kafka-reassignment plan should affirm with following rules:
@@ -62,20 +74,20 @@ def validate_plan(
     - Replication-factor for each partition remains unchanged
     - No duplicate broker-ids in each replicas
     """
-    if not _validate_plan(new_plan):
+    if not _validate_plan(new_plan, allow_rf_mismatch=allow_rf_mismatch):
         _log.error('Invalid proposed-plan.')
         return False
 
     # Validate given plan in reference to base-plan
     if base_plan:
-        if not _validate_plan(base_plan):
+        if not _validate_plan(base_plan, allow_rf_mismatch=allow_rf_mismatch):
             _log.error('Invalid assignment from cluster.')
             return False
         if not _validate_plan_base(
             new_plan,
             base_plan,
             is_partition_subset,
-            allow_rf_change
+            allow_rf_change,
         ):
             return False
     # Plan validation successful
@@ -83,11 +95,11 @@ def validate_plan(
 
 
 def _validate_plan_base(
-    new_plan,
-    base_plan,
-    is_partition_subset=True,
-    allow_rf_change=False,
-):
+    new_plan: PlanDict,
+    base_plan: PlanDict,
+    is_partition_subset: bool = True,
+    allow_rf_change: bool = False,
+) -> bool:
     """Validate if given plan is valid comparing with given base-plan.
 
     Validate following assertions:
@@ -97,14 +109,14 @@ def _validate_plan_base(
     """
 
     # Verify that partitions in plan are subset of base plan.
-    new_partitions = set([
+    new_partitions = {
         (p_data['topic'], p_data['partition'])
         for p_data in new_plan['partitions']
-    ])
-    base_partitions = set([
+    }
+    base_partitions = {
         (p_data['topic'], p_data['partition'])
         for p_data in base_plan['partitions']
-    ])
+    }
     if is_partition_subset:
         invalid_partitions = list(new_partitions - base_partitions)
     else:
@@ -132,7 +144,7 @@ def _validate_plan_base(
     }
     if not allow_rf_change:
         invalid_replication_factor = False
-        for new_partition, replicas in six.iteritems(new_partition_replicas):
+        for new_partition, replicas in new_partition_replicas.items():
             base_replica_cnt = len(base_partition_replicas[new_partition])
             if len(replicas) != base_replica_cnt:
                 invalid_replication_factor = True
@@ -152,7 +164,7 @@ def _validate_plan_base(
     return True
 
 
-def _validate_format(plan):
+def _validate_format(plan: PlanDict) -> bool:
     """Validate if the format of the plan as expected.
 
     Validate format of plan on following rules:
@@ -171,7 +183,7 @@ def _validate_format(plan):
         ]}
     """
     # Verify presence of required keys
-    if set(plan.keys()) != set(['version', 'partitions']):
+    if set(plan.keys()) != {'version', 'partitions'}:
         _log.error(
             'Invalid or incomplete keys in given plan. Expected: "version", '
             '"partitions". Found:{keys}'
@@ -189,40 +201,37 @@ def _validate_format(plan):
 
     # Empty partitions
     if not plan['partitions']:
-        _log.error(
-            '"partitions" list found empty"'
-            .format(version=plan['partitions']),
-        )
+        _log.error('"partitions" list found empty"')
         return False
 
     # Invalid partitions type
     if not isinstance(plan['partitions'], list):
-        _log.error('"partitions" of type list expected.')
+        _log.error('"partitions" of type list expected.')  # type: ignore[unreachable]
         return False
 
     # Invalid partition-data
     for p_data in plan['partitions']:
-        if set(p_data.keys()) != set(['topic', 'partition', 'replicas']):
+        if set(p_data.keys()) != {'topic', 'partition', 'replicas'}:
             _log.error(
                 'Invalid keys in partition-data {keys}'
                 .format(keys=', '.join(list(p_data.keys()))),
             )
             return False
         # Check types
-        if not isinstance(p_data['topic'], six.text_type):
-            _log.error(
+        if not isinstance(p_data['topic'], str):
+            _log.error(  # type: ignore[unreachable]
                 '"topic" of type unicode expected {p_data}, found {t_type}'
                 .format(p_data=p_data, t_type=type(p_data['topic'])),
             )
             return False
         if not isinstance(p_data['partition'], int):
-            _log.error(
+            _log.error(  # type: ignore[unreachable]
                 '"partition" of type int expected {p_data}, found {p_type}'
                 .format(p_data=p_data, p_type=type(p_data['partition'])),
             )
             return False
         if not isinstance(p_data['replicas'], list):
-            _log.error(
+            _log.error(  # type: ignore[unreachable]
                 '"replicas" of type list expected {p_data}, found {r_type}'
                 .format(p_data=p_data, r_type=type(p_data['replicas'])),
             )
@@ -236,7 +245,7 @@ def _validate_format(plan):
         # Invalid broker-type
         for broker in p_data['replicas']:
             if not isinstance(broker, int):
-                _log.error(
+                _log.error(  # type: ignore[unreachable]
                     '"replicas" of type integer list expected {p_data}'
                     .format(p_data=p_data),
                 )
@@ -244,7 +253,7 @@ def _validate_format(plan):
     return True
 
 
-def _validate_plan(plan):
+def _validate_plan(plan: PlanDict, allow_rf_mismatch: bool = False) -> bool:
     """Validate if given plan is valid based on kafka-cluster-assignment protocols.
 
     Validate following parameters:
@@ -263,7 +272,7 @@ def _validate_plan(plan):
         for p_data in plan['partitions']
     ]
     duplicate_partitions = [
-        partition for partition, count in six.iteritems(Counter(partition_names))
+        partition for partition, count in Counter(partition_names).items()
         if count > 1
     ]
     if duplicate_partitions:
@@ -292,18 +301,19 @@ def _validate_plan(plan):
             )
             return False
 
-    # Verify same replication-factor for every topic
-    topic_replication_factor = {}
-    for partition_info in plan['partitions']:
-        topic = partition_info['topic']
-        replication_factor = len(partition_info['replicas'])
-        if topic in list(topic_replication_factor.keys()):
-            if topic_replication_factor[topic] != replication_factor:
-                _log.error(
-                    'Mismatch in replication-factor of partitions for topic '
-                    '{topic}'.format(topic=topic),
-                )
-                return False
-        else:
-            topic_replication_factor[topic] = replication_factor
+    # Verify same replication-factor for partitions in the same topic
+    if not allow_rf_mismatch:
+        topic_replication_factor: dict[str, int] = {}
+        for partition_info in plan['partitions']:
+            topic = partition_info['topic']
+            replication_factor = len(partition_info['replicas'])
+            if topic in list(topic_replication_factor.keys()):
+                if topic_replication_factor[topic] != replication_factor:
+                    _log.error(
+                        'Mismatch in replication-factor of partitions for topic '
+                        '{topic}'.format(topic=topic),
+                    )
+                    return False
+            else:
+                topic_replication_factor[topic] = replication_factor
     return True
