@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 # Copyright 2016 Yelp Inc.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -12,34 +11,68 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
-from __future__ import absolute_import
+from __future__ import annotations
 
 import logging
+import re
+from types import TracebackType
+from typing import Any
+from typing import Callable
+from typing import Sequence
 
-import six
 from kazoo.client import KazooClient
 from kazoo.exceptions import NodeExistsError
 from kazoo.exceptions import NoNodeError
+from kazoo.protocol.states import WatchedEvent
+from kazoo.protocol.states import ZnodeStat
 from kazoo.retry import KazooRetry
+from kazoo.security import ACL
+from typing_extensions import TypedDict
 
+from kafka_utils.util.config import ClusterConfig
 from kafka_utils.util.serialization import dump_json
 from kafka_utils.util.serialization import load_json
 from kafka_utils.util.validation import validate_plan
 
-
 ADMIN_PATH = "/admin"
 REASSIGNMENT_NODE = "reassign_partitions"
 _log = logging.getLogger('kafka-zookeeper-manager')
 
 
+class TopicDataPartitionDict(TypedDict, total=False):
+    replicas: list[int]
+    isr: list[int]
+    controller_epcoh: int
+    leader_epoch: int
+    version: int
+    leader: int
+    ctime: float
+
+
+class TopicDataDict(TypedDict):
+    version: int
+    partitions: dict[str, TopicDataPartitionDict]
+
+
+class ClusterPlanPartitionDict(TypedDict):
+    topic: str
+    partition: int
+    replicas: list[int]
+
+
+class ClusterPlanDict(TypedDict):
+    version: int
+    partitions: list[ClusterPlanPartitionDict]
+
+
 class ZK:
     """Opens a connection to a kafka zookeeper. "
     "To be used in the 'with' statement."""
 
-    def __init__(self, cluster_config):
+    def __init__(self, cluster_config: ClusterConfig) -> None:
         self.cluster_config = cluster_config
 
-    def __enter__(self):
+    def __enter__(self) -> ZK:
         kazooRetry = KazooRetry(
             max_tries=5,
         )
@@ -55,48 +88,53 @@ class ZK:
         self.zk.start()
         return self
 
-    def __exit__(self, type, value, traceback):
+    def __exit__(self, type: type | None, value: BaseException, traceback: TracebackType) -> None:
         self.zk.stop()
 
-    def get_children(self, path, watch=None):
+    def get_children(self, path: str, watch: Callable[[WatchedEvent], None] | None = None) -> list[str]:
         """Returns the children of the specified node."""
         _log.debug(
-            "ZK: Getting children of {path}".format(path=path),
+            f"ZK: Getting children of {path}",
         )
         return self.zk.get_children(path, watch)
 
-    def get(self, path, watch=None):
+    def get(self, path: str, watch: Callable[[WatchedEvent], None] | None = None) -> tuple[bytes, ZnodeStat]:
         """Returns the data of the specified node."""
         _log.debug(
-            "ZK: Getting {path}".format(path=path),
+            f"ZK: Getting {path}",
         )
         return self.zk.get(path, watch)
 
-    def set(self, path, value):
+    def set(self, path: str, value: bytes) -> ZnodeStat:
         """Sets and returns new data for the specified node."""
         _log.debug(
-            "ZK: Setting {path} to {value}".format(path=path, value=value)
+            f"ZK: Setting {path} to {value!r}"
         )
         return self.zk.set(path, value)
 
-    def get_json(self, path, watch=None):
+    def get_json(self, path: str, watch: Callable[[WatchedEvent], None] | None = None) -> Any:
         """Reads the data of the specified node and converts it to json."""
         data, _ = self.get(path, watch)
         return load_json(data) if data else None
 
-    def get_broker_metadata(self, broker_id):
+    def get_broker_metadata(self, broker_id: str) -> dict[str, Any]:
         try:
-            broker_json, _ = self.get(
-                "/brokers/ids/{b_id}".format(b_id=broker_id)
-            )
+            broker_json = load_json(self.get(
+                f"/brokers/ids/{broker_id}"
+            )[0])
+            if (broker_json['host'] is None):
+                pattern = '(?:[SSL|INTERNAL|PLAINTEXTSASL].*://)?(?P<host>[^:/ ]+).?(?P<port>[0-9]*).*'
+                result = re.search(pattern, broker_json['endpoints'][0])
+                assert result is not None
+                broker_json['host'] = result.group('host')
         except NoNodeError:
             _log.error(
-                "broker '{b_id}' not found.".format(b_id=broker_id),
+                f"broker '{broker_id}' not found.",
             )
             raise
-        return load_json(broker_json)
+        return broker_json
 
-    def get_brokers(self, names_only=False):
+    def get_brokers(self, names_only: bool = False) -> dict[int, dict[str, Any] | None]:
         """Get information on all the available brokers.
 
         :rtype : dict of brokers
@@ -113,35 +151,18 @@ class ZK:
             return {int(b_id): None for b_id in broker_ids}
         return {int(b_id): self.get_broker_metadata(b_id) for b_id in broker_ids}
 
-    def get_topic_config(self, topic):
+    def get_topic_config(self, topic: str) -> dict[str, Any]:
         """Get configuration information for specified topic.
 
         :rtype : dict of configuration
         """
-        try:
-            config_data = load_json(
-                self.get(
-                    "/config/topics/{topic}".format(topic=topic)
-                )[0]
-            )
-        except NoNodeError as e:
+        return self._get_entity_config(
+            "topics",
+            topic,
+            lambda topic: len(self.get_topics(topic_name=topic, fetch_partition_state=False)) > 0,
+        )
 
-            # Kafka version before 0.8.1 does not have "/config/topics/<topic_name>" path in ZK and
-            # if the topic exists, return default dict instead of raising an Exception.
-            # Ref: https://cwiki.apache.org/confluence/display/KAFKA/Kafka+data+structures+in+Zookeeper.
-
-            topics = self.get_topics(topic_name=topic, fetch_partition_state=False)
-            if len(topics) > 0:
-                _log.info("Configuration not available for topic {topic}.".format(topic=topic))
-                config_data = {"config": {}}
-            else:
-                _log.error(
-                    "topic {topic} not found.".format(topic=topic)
-                )
-                raise e
-        return config_data
-
-    def set_topic_config(self, topic, value, kafka_version=(0, 10, )):
+    def set_topic_config(self, topic: str, value: str, kafka_version: tuple[int, int] = (0, 10, )) -> None:
         """Set configuration information for specified topic.
 
         :topic : topic whose configuration needs to be changed
@@ -152,34 +173,104 @@ class ZK:
             Defaults to (0, 10, x). Kafka version 9 and kafka 10
             support this feature.
         """
+        self._set_entity_config("topics", topic, value, kafka_version)
+
+    def get_broker_config(self, broker_id: str) -> dict[str, Any]:
+        """Get configuration information for specified broker.
+
+        :rtype : dict of configuration
+        """
+        return self._get_entity_config(
+            "brokers",
+            broker_id,
+            lambda broker_id: broker_id in self.get_brokers(names_only=True)
+        )
+
+    def set_broker_config(self, broker_id: str, value: str, kafka_version: tuple[int, int] = (0, 10, )) -> ZnodeStat:
+        """Set configuration information for specified broker.
+
+        :broker_id : broker whose configuration needs to be changed
+        :value :  config value with which the topic needs to be
+            updated with. This would be of the form key=value.
+            Example 'cleanup.policy=compact'
+        :kafka_version :tuple kafka version the brokers are running on.
+            Defaults to (0, 10, x). Versions above Kafka 0.9 support this feature.
+        """
+        return self._set_entity_config("brokers", broker_id, value, kafka_version)
+
+    def _get_entity_config(self, entity_type: str, entity_name: str, entity_exists: Callable[[str], bool]) -> dict[str, Any]:
+        """Get configuration information for specified broker.
+
+        :entity_type : "brokers" or "topics"
+        :entity_name : broker id or topic name
+        :entity_exists : fn(entity_name) -> bool to determine whether an entity
+                            exists. used to determine whether to throw an exception
+                            when a configuration cannot be found for the given entity_name
+        :rtype : dict of configuration
+        """
+        assert entity_type in ("brokers", "topics"), "Supported entities are brokers and topics"
+
+        try:
+            config_data = load_json(
+                self.get(
+                    f"/config/{entity_type}/{entity_name}"
+                )[0]
+            )
+        except NoNodeError as e:
+            if entity_exists(entity_name):
+                _log.info("Configuration not available for {entity_type} {entity_name}.".format(
+                    entity_type=entity_type,
+                    entity_name=entity_name,
+                ))
+                config_data = {"config": {}}
+            else:
+                _log.error(f"{entity_type} {entity_name} not found")
+                raise e
+
+        return config_data
+
+    def _set_entity_config(self, entity_type: str, entity_name: str, value: str, kafka_version: tuple[int, int] = (0, 10, )) -> ZnodeStat:
+        """Set configuration information for specified entity.
+
+        :entity_type : "brokers" or "topics"
+        :entity_name : broker id or topic name
+        :value :  config value with which the entity needs to be
+            updated with. This would be of the form key=value.
+            Example 'cleanup.policy=compact'
+        :kafka_version :tuple kafka version the brokers are running on.
+            Defaults to (0, 10, x). Versions above Kafka 0.9 support this feature.
+        """
+        assert entity_type in ("brokers", "topics"), "Supported entities are brokers and topics"
+
         config_data = dump_json(value)
 
         try:
             # Change value
             return_value = self.set(
-                "/config/topics/{topic}".format(topic=topic),
-                config_data
+                f"/config/{entity_type}/{entity_name}",
+                config_data,
             )
+
             # Create change
-            version = kafka_version[1]
+            assert kafka_version >= (0, 9, ), "Feature supported with kafka 0.9 and above"
 
-            # this feature is supported in kafka 9 and kafka 10
-            assert version in (9, 10), "Feature supported with kafka 9 and kafka 10"
-
-            if version == 9:
+            if kafka_version < (0, 10, ):
                 # https://github.com/apache/kafka/blob/0.9.0.1/
                 #     core/src/main/scala/kafka/admin/AdminUtils.scala#L334
                 change_node = dump_json({
                     "version": 1,
-                    "entity_type": "topics",
-                    "entity_name": topic
+                    "entity_type": entity_type,
+                    "entity_name": entity_name,
                 })
-            else:  # kafka 10
+            else:  # kafka 0.10+
                 # https://github.com/apache/kafka/blob/0.10.2.1/
                 #     core/src/main/scala/kafka/admin/AdminUtils.scala#L574
                 change_node = dump_json({
                     "version": 2,
-                    "entity_path": "topics/" + topic,
+                    "entity_path": "{entity_type}/{entity_name}".format(
+                        entity_type=entity_type,
+                        entity_name=entity_name
+                    )
                 })
 
             self.create(
@@ -189,17 +280,26 @@ class ZK:
             )
         except NoNodeError as e:
             _log.error(
-                "topic {topic} not found.".format(topic=topic)
+                f"{entity_type}: {entity_name} not found."
             )
             raise e
         return return_value
 
     def get_topics(
         self,
-        topic_name=None,
-        names_only=False,
-        fetch_partition_state=True,
-    ):
+        topic_name: str | None = None,
+        names_only: bool = False,
+        fetch_partition_state: bool = True,
+    ) -> dict[str, TopicDataDict] | list[str]:
+        topic_names = [topic_name] if topic_name else None
+        return self.get_multiple_topics(topic_names, names_only, fetch_partition_state)
+
+    def get_multiple_topics(
+        self,
+        topic_names: list[str] | None = None,
+        names_only: bool = False,
+        fetch_partition_state: bool = True,
+    ) -> dict[str, TopicDataDict] | list[str]:
         """Get information on all the available topics.
 
         Topic-data format with fetch_partition_state as False :-
@@ -233,9 +333,8 @@ class ZK:
         required fetch_partition_state should be set to False.
         """
         try:
-            topic_ids = [topic_name] if topic_name else self.get_children(
-                "/brokers/topics",
-            )
+            if not topic_names:
+                topic_names = self.get_children("/brokers/topics")
         except NoNodeError:
             _log.error(
                 "Cluster is empty."
@@ -243,22 +342,22 @@ class ZK:
             return {}
 
         if names_only:
-            return topic_ids
+            return topic_names
         topics_data = {}
-        for topic_id in topic_ids:
+        for topic_id in topic_names:
             try:
-                topic_info = self.get("/brokers/topics/{id}".format(id=topic_id))
+                topic_info = self.get(f"/brokers/topics/{topic_id}")
                 topic_data = load_json(topic_info[0])
                 topic_ctime = topic_info[1].ctime / 1000.0
                 topic_data['ctime'] = topic_ctime
             except NoNodeError:
                 _log.info(
-                    "topic '{topic}' not found.".format(topic=topic_id),
+                    f"topic '{topic_id}' not found.",
                 )
                 return {}
             # Prepare data for each partition
-            partitions_data = {}
-            for p_id, replicas in six.iteritems(topic_data['partitions']):
+            partitions_data: dict[str, TopicDataPartitionDict] = {}
+            for p_id, replicas in topic_data['partitions'].items():
                 partitions_data[p_id] = {}
                 if fetch_partition_state:
                     # Fetch partition-state from zookeeper
@@ -274,7 +373,7 @@ class ZK:
             topics_data[topic_id] = topic_data
         return topics_data
 
-    def get_consumer_groups(self, consumer_group_id=None, names_only=False):
+    def get_consumer_groups(self, consumer_group_id: str | None = None, names_only: bool = False) -> dict[str, dict[str, dict[str, Any]] | None]:
         """Get information on all the available consumer-groups.
 
         If names_only is False, only list of consumer-group ids are sent.
@@ -305,12 +404,12 @@ class ZK:
         if names_only:
             return {g_id: None for g_id in group_ids}
 
-        consumer_offsets = {}
+        consumer_offsets: dict[str, dict[str, dict[str, Any]] | None] = {}
         for g_id in group_ids:
             consumer_offsets[g_id] = self.get_group_offsets(g_id)
         return consumer_offsets
 
-    def get_group_offsets(self, group, topic=None):
+    def get_group_offsets(self, group: str, topic: str | None = None) -> dict[str, dict[str, Any]]:
         """Fetch group offsets for given topic and partition otherwise all topics
         and partitions otherwise.
 
@@ -324,7 +423,7 @@ class ZK:
             }
         }
         """
-        group_offsets = {}
+        group_offsets: dict[str, dict[str, Any]] = {}
         try:
             all_topics = self.get_my_subscribed_topics(group)
         except NoNodeError:
@@ -372,11 +471,11 @@ class ZK:
                     offset_json, _ = self.get(path)
                     group_offsets[topic][partition] = load_json(offset_json)
                 except NoNodeError:
-                    _log.error("Path {path} not found".format(path=path))
+                    _log.error(f"Path {path} not found")
                     raise
         return group_offsets
 
-    def _fetch_partition_state(self, topic_id, partition_id):
+    def _fetch_partition_state(self, topic_id: str, partition_id: str) -> Any:
         """Fetch partition-state for given topic-partition."""
         state_path = "/brokers/topics/{topic_id}/partitions/{p_id}/state"
         try:
@@ -387,7 +486,7 @@ class ZK:
         except NoNodeError:
             return {}  # The partition has no data
 
-    def _fetch_partition_info(self, topic_id, partition_id):
+    def _fetch_partition_info(self, topic_id: str, partition_id: str) -> Any:
         """Fetch partition info for given topic-partition."""
         info_path = "/brokers/topics/{topic_id}/partitions/{p_id}"
         try:
@@ -398,17 +497,17 @@ class ZK:
         except NoNodeError:
             return {}  # The partition has no data
 
-    def get_my_subscribed_topics(self, groupid):
+    def get_my_subscribed_topics(self, groupid: str) -> list[str]:
         """Get the list of topics that a consumer is subscribed to
 
         :param: groupid: The consumer group ID for the consumer
         :returns list of kafka topics
         :rtype: list
         """
-        path = "/consumers/{group_id}/offsets".format(group_id=groupid)
+        path = f"/consumers/{groupid}/offsets"
         return self.get_children(path)
 
-    def get_my_subscribed_partitions(self, groupid, topic):
+    def get_my_subscribed_partitions(self, groupid: str, topic: str) -> list[str]:
         """Get the list of partitions of a topic
         that a consumer is subscribed to
 
@@ -423,9 +522,9 @@ class ZK:
         )
         return self.get_children(path)
 
-    def get_cluster_assignment(self):
+    def get_cluster_assignment(self, topic_names: list[str] | None = None) -> dict[tuple[str, int], list[int]]:
         """Fetch the cluster layout in form of assignment from zookeeper"""
-        plan = self.get_cluster_plan()
+        plan = self.get_cluster_plan(topic_names)
         assignment = {}
         for elem in plan['partitions']:
             assignment[
@@ -436,13 +535,13 @@ class ZK:
 
     def create(
         self,
-        path,
-        value='',
-        acl=None,
-        ephemeral=False,
-        sequence=False,
-        makepath=False
-    ):
+        path: str,
+        value: bytes = b'',
+        acl: Sequence[ACL] | None = None,
+        ephemeral: bool = False,
+        sequence: bool = False,
+        makepath: bool = False,
+    ) -> str:
         """Creates a Zookeeper node.
 
         :param: path: The zookeeper node path
@@ -458,7 +557,7 @@ class ZK:
         _log.debug("ZK: Creating node " + path)
         return self.zk.create(path, value, acl, ephemeral, sequence, makepath)
 
-    def delete(self, path, recursive=False):
+    def delete(self, path: str, recursive: bool = False) -> bool:
         """Deletes a Zookeeper node.
 
         :param: path: The zookeeper node path
@@ -467,27 +566,30 @@ class ZK:
         _log.debug("ZK: Deleting node " + path)
         return self.zk.delete(path, recursive=recursive)
 
-    def delete_topic(self, groupid, topic):
+    def delete_topic(self, groupid: str, topic: str) -> None:
         path = "/consumers/{groupid}/offsets/{topic}".format(
             groupid=groupid,
             topic=topic,
         )
         self.delete(path, True)
 
-    def delete_group(self, groupid):
+    def delete_group(self, groupid: str) -> None:
         path = "/consumers/{groupid}".format(
             groupid=groupid,
         )
         self.delete(path, True)
 
-    def execute_plan(self, plan, allow_rf_change=False):
+    def execute_plan(self, plan: ClusterPlanDict, allow_rf_change: bool = False, allow_rf_mismatch: bool = False) -> bool:
         """Submit reassignment plan for execution."""
         reassignment_path = '{admin}/{reassignment_node}'\
             .format(admin=ADMIN_PATH, reassignment_node=REASSIGNMENT_NODE)
         plan_json = dump_json(plan)
-        base_plan = self.get_cluster_plan()
-        if not validate_plan(plan, base_plan, allow_rf_change=allow_rf_change):
-            _log.error('Given plan is invalid. Aborting new reassignment plan ... {plan}'.format(plan=plan))
+        topic_names_from_proposed_plan = set()
+        for partition in plan['partitions']:
+            topic_names_from_proposed_plan.add(partition['topic'])
+        base_plan = self.get_cluster_plan(topic_names=list(topic_names_from_proposed_plan))
+        if not validate_plan(plan, base_plan, allow_rf_change=allow_rf_change, allow_rf_mismatch=allow_rf_mismatch):
+            _log.error(f'Given plan is invalid. Aborting new reassignment plan ... {plan}')
             return False
         # Send proposed-plan to zookeeper
         try:
@@ -500,7 +602,7 @@ class ZK:
             return True
         except NodeExistsError:
             _log.warning('Previous plan in progress. Exiting..')
-            _log.warning('Aborting new reassignment plan... {plan}'.format(plan=plan))
+            _log.warning(f'Aborting new reassignment plan... {plan}')
             in_progress_plan = load_json(self.get(reassignment_path)[0])
             in_progress_partitions = [
                 '{topic}-{p_id}'.format(
@@ -526,27 +628,28 @@ class ZK:
             )
             return False
 
-    def get_cluster_plan(self):
+    def get_cluster_plan(self, topic_names: list[str] | None = None) -> ClusterPlanDict:
         """Fetch cluster plan from zookeeper."""
 
         _log.info('Fetching current cluster-topology from Zookeeper...')
-        cluster_layout = self.get_topics(fetch_partition_state=False)
+        cluster_layout = self.get_multiple_topics(topic_names, fetch_partition_state=False)
+        assert isinstance(cluster_layout, dict)
         # Re-format cluster-layout
-        partitions = [
+        partitions: list[ClusterPlanPartitionDict] = [
             {
                 'topic': topic_id,
                 'partition': int(p_id),
                 'replicas': partitions_data['replicas']
             }
-            for topic_id, topic_info in six.iteritems(cluster_layout)
-            for p_id, partitions_data in six.iteritems(topic_info['partitions'])
+            for topic_id, topic_info in cluster_layout.items()
+            for p_id, partitions_data in topic_info['partitions'].items()
         ]
         return {
             'version': 1,
             'partitions': partitions
         }
 
-    def get_pending_plan(self):
+    def get_pending_plan(self) -> Any:
         """Read the currently running plan on reassign_partitions node."""
         reassignment_path = '{admin}/{reassignment_node}'\
             .format(admin=ADMIN_PATH, reassignment_node=REASSIGNMENT_NODE)
